/***************************************************************************
                          arc_tgz.cpp  -  description
                             -------------------
    begin                : Sun Aug 20 2000
    copyright            : (C) 2000 by Morten Poulsen
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "arc_tgz.h"
#include "sc_archive.h"


//////////////////////TAR.GZ/////////////////////////////


    bool cFileInTar::Open(tOpenMode om) {
      if (om==omWrite || om==omGZWrite) return false;
      bool getheader=true;
      tar_buffer tbuf;
      gzf=GetFilePointer(spd.fullname);
      if (!gzf->Open(omGZRead)) {scDELETE(gzf); return false;};
      do {
        ShowRotator();
        int i=gzf->Read(&tbuf, _BLOCKSIZE);
        if (i<0) {scDELETE(gzf); return false;};            //Read error
        if (i!=_BLOCKSIZE && i!=0) {scDELETE(gzf); return false;};  //Incomplete block read
        if (i==0) break;
        if (getheader) {
          string s=tbuf.header.name;
          if (s.length()==0) continue;
          #ifdef _TARGETFLAVOR_UNIX_
            filemode=(int)getoct(tbuf.header.mode,8);
          #endif
      
          //This is nessecary to know how many block to skip
          filesize=(time_t)getoct(tbuf.header.size,12);
          switch (tbuf.header.typeflag) {
            case DIRTYPE : {
              getheader=true;
              break;
            }
            case REGTYPE :
            case AREGTYPE : {
              remaining=filesize;
              if(s==spd.arc_fullname) {
                return true;
              }
              getheader=(remaining==0);
              break;
            }
            default : {
              getheader=true;
            }
          }
        } else {
          unsigned int bytes=(remaining>_BLOCKSIZE) ? _BLOCKSIZE : remaining;
          remaining-=bytes;
          if (remaining==0) getheader=true;
        }
      } while (42);
      scDELETE(gzf);
      return false;
    }

    int cFileInTar::Read(void *buf,int size) {
      if (remaining==0) return 0;
      if (gzf==NULL) return -1;

      int i=gzf->Read(buf, _BLOCKSIZE);
      if (i<0) {scDELETE(gzf); return false;};            //Read error
      if (i!=_BLOCKSIZE && i!=0) {scDELETE(gzf); return false;};  //Incomplete block read
      //if (i==0 || tbuf.header.name[0]==0) break;
      if (remaining<_BLOCKSIZE) {
        int i=remaining;
        remaining=0;
        scDELETE(gzf);
        return i;
      } else {
        remaining-=_BLOCKSIZE;
        return _BLOCKSIZE;
      }
    }
    

int getoct(char *p,int width) {
  int result = 0; char c;
  while (width --) {
    c = *p++;
    if (c == ' ')	continue;
    if (c == 0)	break;
    result = result * 8 + (c - '0');
  }
  return result;
}



bool TarScan(tDirContainer& cont, const sParsedDir& spd, bool onlydirs/*=false*/) {
  bool getheader=true;
  bool nodirrecs=true; //Bad form of tar.gz-files
  int remaining=0;
  tar_buffer tbuf;
  cFile* tarf=GetFilePointer(spd.fullname);
  if (!tarf->Open(omGZRead)) {delete tarf; return false;};
  cont.Clear();
  if (!onlydirs) Add2Dots(cont);

  cDirEntry tde; tde.isdir=false;
  do {
    ShowRotator();
    int i=tarf->Read(&tbuf, _BLOCKSIZE);
    if (i<0) {delete tarf; return false;};            //Read error
    if (i!=_BLOCKSIZE && i!=0) {delete tarf; return false;};  //Incomplete block read
    if (i==0) break;
    if (getheader) {
      string s=tde.name=tbuf.header.name;
      bool addfile;
      if (addfile=FixDirsInArchive(tde.name,spd)) {
        //Don't use time on this if it should not be used anyway
        if (s.length()>0 && s!=".") {
          tde.MakeNormal(tde.name,("{"+spd.fullname+"}"+s), useMode,
                         (int)getoct(tbuf.header.mode,8),
                         (time_t)getoct(tbuf.header.mtime,12),
                         (int)getoct(tbuf.header.size,12));
          } else addfile=false;          
      } else {
        //This is nessecary to know how many block to skip
        tde.size=(int)getoct(tbuf.header.size,12);
      }

      switch (tbuf.header.typeflag) {
        case DIRTYPE : {
          tde.isdir=true;
          getheader=true;
          break;
        }
        case REGTYPE :
        case AREGTYPE : {
          tde.isdir=false;
          remaining=tde.size;
          getheader=(remaining==0);
          break;
        }
        default : {
          getheader=true;
        }
      }

      /*Handle the ugly archives with no dirrecs*/
      if (tde.isdir) nodirrecs=false; //Hey, we've got dirrecs!
      if (!tde.isdir  //if we have a file
         && nodirrecs //and no dirrecs
         && (s.find('/')!=-1))  //and it's not in the root of the archive
           CheckForMissingDirRecord(cont,s,spd, useMode); //Check and perhaps insert proper dir
      /*********************************/

      if (addfile) { //Only add if confirmed previously
        if (onlydirs) {
           if (tde.isdir) cont+=tde;
        } else cont+=tde;
      }
    } else {
      unsigned int bytes=(remaining>_BLOCKSIZE) ? _BLOCKSIZE : remaining;
      remaining-=bytes;
      //if (remaining==0) getheader=true;
      getheader=(remaining==0);      
    }    
  } while (42);
  delete tarf;
  return true;
}


bool UnpackTar(const string& filename, const string& destdir, tProgFunc progress) {
  bool getheader=true;
  int remaining=0;
  tar_buffer tbuf;
  cFile* tarf=GetFilePointer(filename);
  if (!tarf->Open(omGZRead)) {delete tarf; return false;};
  cFile* outf=NULL;
  do {
    int i=tarf->Read(&tbuf, _BLOCKSIZE);
    if (i<0) {delete tarf; return false;};            //Read error
    if (i!=_BLOCKSIZE && i!=0) {delete tarf; return false;};  //Incomplete block read
    if (i==0) break;
    if (getheader) {
      string s=tbuf.header.name;
      if (s.length()==0 || s==".") continue;
      //This is nessecary to know how many block to skip
      long filesize=(int)getoct(tbuf.header.size,12);
      switch (tbuf.header.typeflag) {
        case DIRTYPE : {
          MkDir(destdir+s);
          getheader=true;
          break;
        }
        case REGTYPE :
        case AREGTYPE : {
          remaining=filesize;
          getheader=(remaining==0);
          MakeComplexDir(destdir+s, true);
          outf=GetFilePointer(destdir+s);
          if (!outf->Open(omWrite)) {
            delete tarf; return false;
          }
          break;
        }
        default : {
          getheader=true;
        }
      }
    } else {
      //if more than BLOCKSIZE is remaining just use a BLOCKSIZE, else use remaining
      unsigned int bytes=(remaining>_BLOCKSIZE) ? _BLOCKSIZE : remaining;
      remaining-=bytes;
      if (outf!=NULL) {
        outf->Write(&tbuf,bytes);
        progress(tarf->Pos(), tarf->Size(),0);
      }
      if (remaining==0) {
        getheader=true;
        outf->Close();
        scDELETE(outf);
      }
    }
    
  } while (42);
  delete tarf;
  return true;
}


