/***************************************************************************
              array.h  -  templatebased array class with sorting
                             -------------------
    begin                : Thu Dec 23 1999
    copyright            : (C) 1999 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//Version 0.4.7

/* History:
 0.1   - First work version
 0.2   - Beginning to implement sorted version REMOVED temporarily
 0.2.2 - Assignment operator (Clone method), which physically copies the whole
         darn thing
 0.3   - Optimized with array of pointers
 0.4   - Added Insert-method
 0.4.1 - Fixed Copy constructor / assignmentoperator
 0.4.2 - Made swap method, should accelerate sorting
 0.4.3 - Corrected a nasty bug in the copy constructor
 0.4.4 - Fixed yet another bug in Clear-method, cleaned the sourced up
 0.4.5 - PopSome which returns any record and deletes it from array
 0.4.6 - Made Last/First, phew that was difficult
 0.4.7 - Begun adding standard STL-methods for combatibility

*/

//Preconditions: Working Copy-contructor (default or specific)
#ifndef __ARRAY_H_
#define __ARRAY_H_

const long int _defaultsize = 128;

template<class TYPE>
class cArray {
  public:
    cArray(long int max=_defaultsize) {
      Initialize(max);
    };

    cArray(const cArray& other) {
      Initialize(other.Size());
      for(long int li=0; li<maxitems; li++) AddItem(other.Read(li));
    }

    cArray& operator=(const cArray& other) {
      Destroy();
      nextfree=0; firstnext=0;
      maxitems=other.Size();
      entries=new cEntry*[maxitems];
      for(long int li=0; li<maxitems; li++) AddItem(other.Read(li));
    }

    ~cArray() {
      Destroy();
    }

    TYPE Read(long int index) const {
      if (index<0) return Read(0);
      if (index>=nextfree) return Read(index % nextfree);
      return entries[index]->data;
    }

    TYPE& GetItem(long int index) {
      if (index<0) return GetItem(0);
      if (index>nextfree-1) return GetItem(index % nextfree);
      return entries[index]->data;
    };

    TYPE& operator[] (long int index) {return GetItem(index);};

    TYPE& First() {
      if (nextfree!=0) {
        return GetItem(0);
      }
    }

    TYPE& Last() {
      if (nextfree!=0) {
        return GetItem(Size()-1);
        //return entries[nextfree-1]->data;
      }
    }

    bool Swap(long int index1, long int index2) {
      if (index1<0 || index1>=nextfree) return false;
      if (index2<0 || index2>=nextfree) return false;
      if (index1==index2) return true; //or false?
      cEntry *temp=entries[index1];
      entries[index1]=entries[index2];

      entries[index2]=temp;
    }

    void Clear() {
      Destroy();
      Initialize();
    }

    bool Empty() {return (nextfree==0);};

    bool First(TYPE& e) {
      if (Empty()) return false;
      firstnext=1;
      entries[0]->data;
      return true;
    }

    bool Next(TYPE& e) {
      if (firstnext<nextfree) {
        e=entries[firstnext]->data;
        firstnext++;
        return true;
      } else {
        return false;
      }
    }    

    //for STL-compatibility
    void push_back(const TYPE& newitem) {AddItem(newitem);};
    void AddItem(const TYPE& newitem) {
      if (nextfree>maxitems) Expand();
      entries[nextfree]=new cEntry(newitem);
      nextfree++;
    }
    void PushEnd(const TYPE& newitem) {
      AddItem(newitem);
    }

    void operator+= (const TYPE& newitem) {AddItem(newitem);};
    void operator+= (const cArray& other) {

    };

    void push_front(const TYPE& newitem) {Insert(newitem,0);};
    void PushStart(const TYPE& newitem)  {Insert(newitem,0);};

    TYPE PopStart() {
      TYPE temp=GetItem(0);
      RemoveItem(0);
      return temp;
    }

    TYPE PopEnd() {
      TYPE temp=GetItem(nextfree-1);
      RemoveItem(nextfree-1);
      return temp;
    }

    TYPE PopSome(long int index) {
      TYPE temp=GetItem(index);
      RemoveItem(index);
      return temp;
    }

    //for STL-compatibility
    void insert(long int index, const TYPE& newitem) {Insert(newitem,index);};
    void Insert(const TYPE& newitem, long int index) {
      if (index>maxitems) {
        Expand(index+50);
      } else {
        if (nextfree>maxitems) Expand();
      } //now we are certain there are room
      for(long int li=nextfree; li>index; li--) {
        entries[li]=entries[li-1]; //move all after index 1 place forward
      }
      entries[index]=new cEntry(newitem);
      nextfree++;
    }

    //for STL-compatibility
    //void erase(long int index) {RemoveItem(index);};
    bool RemoveItem(long int index) {
      if (index<0) return false;
      if (index>=nextfree) return false;
      delete entries[index];
      nextfree--;
      if (index==nextfree) return true; //it was the last item
      for(long int li=index; li<nextfree; li++) {
        entries[li]=entries[li+1]; //move all items back in array
      }

      return true;
    }
    bool operator-= (long int index) {return RemoveItem(index);};
    bool operator== (const cArray& other) {
      return other.Size()=this.Size();
      //Is this what comparison should do?
    };

    long int Size() const {return nextfree;};
    //for STL-compatibility
    long int size() const {return nextfree;};
  protected:
    class cEntry {
    public:
      cEntry(const TYPE& indata) {
        data=indata;
      }  
      TYPE data;
    };

  
    cEntry **entries;
    long int nextfree;
    long int maxitems;
    long int firstnext;
    void Expand(long int newmax=-1) {
      if (newmax==-1) newmax=maxitems*2;
      cEntry **old=entries;
      entries=new cEntry*[newmax];
      for (long int li=0; li<nextfree; li++) {
        entries[li]=old[li];
      }
      for (long int li=nextfree; li<newmax; li++) {
        entries[li]=0; //NULL
      }
      maxitems=newmax;
    }

    void Initialize(long int max=_defaultsize) {
      maxitems=max; nextfree=0; firstnext=0;
      entries=new cEntry*[maxitems];
      for(long int li=0; li<maxitems; li++) entries[li]=0; //NULL
    }

    void Destroy() {
      delete []entries;
    }
};

#endif //__ARRAY_H_


