/***************************************************************************
                          fm_console.cpp  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fm_defs.h"
#include "fm_str.h"
#include "fm_console.h"
#include "fm_conio.h"

cConsole::cConsole(int nr)  {
  Dbg("fm_console: Initialising console ",nr);
  connr=nr;
  mode=cmDir;
  dir=new cDirWindow();
  view=NULL;
  edit=NULL;
}

cConsole::~cConsole()  {
  Dbg("fm_console: Destroying console");
  delete dir;
  if (view!=NULL) scDELETE(view);
  if (edit!=NULL) scDELETE(edit);
};

void cConsole::TreatKey(tKey& k) {
  Wr(1,1,MkStr(connr)+" ",chYELLOW,cBLUE);
  switch (mode) {
    case cmDir : {
      switch (k) {
        case kF3 : //not break ;-) ==kV //fallthrough intentional ;-)
        case kV : {
          view=new cViewWindow(dir->GetCurrentFile());
          if (view->Error()) {
            scDELETE(view);
            Update();
          } else {
            mode=cmView;
          }
          break;
        }

        case kN :
        case kShiftF9 :
        case kShiftF8 :
        case kF8 :
        case kF9 : {
          if (KeyIn(k,kF8,kF9)) {
            if (k==kF8) {
              DedicatedEdit(dir->GetCurrentFile(),true);    //Use external
              Update();
              break;
            } else {
              edit=new cEditWindow(dir->GetCurrentFile()); //Use internal
            }
          } else {
            if (k==kShiftF8) {
              DedicatedEdit("",true);
              Update();
              break;
            } else {
              edit=new cEditWindow();
              edit->SetPath(dir->GetFullPath());
            }
          }
          if (edit->Error()) {
            scDELETE(edit);
            Update();
          } else {
            mode=cmEdit;
          }
          break;
        }

        default : dir->TreatKey(k);
      }
      Wr(1,1,MkStr(connr)+" ",chYELLOW,cBLUE);
      break;
    }
    case cmView : {
      switch (k) {
        case kQ :
        case kAltX :
        case kESC : //fallthrough intentional ;-)
        case kF10 : {
          scDELETE(view);
          mode=cmDir; Update();
          k=kDummyKey; //only exit editor, not completely
          break;
        }

        default: {
          view->TreatKey(k);
        }
      }
      break;
    }
    case cmEdit : {
      switch (k) {
        case kAltX :
        case kESC : //fallthrough intentional ;-)
        case kF10 : {
          if (edit->ConfirmExit()) {
            scDELETE(edit); mode=cmDir; Update();
          }
          break;
        }

        default: {
          edit->TreatKey(k);
        }
        k=kDummyKey; //only exit editor, not completely
      }
      break;
    }
  }
}

void cConsole::Idle() {
  switch (mode) {
    case cmView : view->Idle(); break;
    case cmEdit : edit->Idle(); break;
    default : dir->Idle();
  }
}

bool cConsole::ConfirmExit() {
  if (mode==cmEdit) return edit->ConfirmExit(); else return true;
}

void cConsole::Update() {
  Wr(1,1,MkStr(connr)+" ",chYELLOW,cBLUE);
  switch (mode) {
    case cmView : view->Update(); break;
    case cmEdit : edit->Update(); break;
    default : dir->Update();
  }
}

string cConsole::GetTitle() {
  switch (mode) {
    cmView : return view->GetTitle();
    cmEdit : return edit->GetTitle();
    default : return dir->GetTitle();
  }
}

