/***************************************************************************
                          fm_init.cpp  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <fstream>
#include "fm_init.h"
#include "fm_defs.h"
#include "fm_conio.h"
#include "fm_keyboard.h"
#include "fm_debug.h"
#include "target.h"
#include "fm_conf.h"
#include "stopwatch.h"
#include "fm_window.h"
#include "sc_signals.h"
#include "fm_vfs.h"
#include "fm_editcon.h"

string _sc_outdir="";

cStopwatch st;

void InitSystem() {
  if (IsDebuggingEnabled()) cout <<"Debugging enabled\n";

  cout <<"Initializing console driver... ";
  InitConIO();
  cout <<"Done!" <<endl;

  cout <<"Initializing keyboard driver... ";
  InitKeyboard();
  cout <<"Done!" <<endl;

  cout <<"Setting up signal handlers... ";
  SetupSignalHandlers();
  cout <<"Done!" <<endl;
}

void StartUpSequence(int argc, char* argv[]) {
  cout <<"Sunshine Commander - " <<_VERSION <<endl;
  cout <<"Built for " <<_TARGET_DESCRIPTION_ <<endl;
  InitDebugging();

  //Handle Configuration file
  Configuration.ReadConfigFile();

  //Handle commandline parameters
  for(int i=1; i<argc; i++) { //0 = complete path of the running program
    string s=argv[i];
    if (DoesDirExist(s)) {
      cout <<"Setting initial dir: " <<s <<endl;
      SetDir(s);
      continue;
    };
    
    if (s[1]=='e' || s=="--edit") {
      if (i==(argc-1)) {
        cout <<"Usage scedit -e [filename]" <<endl;
        //ShutDownSequence(false);
        exit(1);
      }
      string name=argv[i+1];
      if (!DoesFileExist(name)) {
        cout <<"File does not exist" <<endl;
        //ShutDownSequence(false);
        exit(1);
      }
      InitSystem();
      DedicatedEdit(name);
      ShutDownSequence();
      exit(0);
    }
    if (s[1]=='d' || s=="--debug") {
      EnableDebugging();
      continue;
    }
    if (s[1]=='h' || s=="--help") {
      cout <<endl <<"Usage: sc [options] [dir to start in]" <<endl;
      cout <<       "   or: scedit filename" <<endl;
      cout <<endl <<"Options:" <<endl;
      cout <<"  -d or --debug         : Enable debuglog generated as ";
      #ifdef _TARGET_UNIX_
        cout <<"$HOME/scdebug.dbg";
      #else
        cout <<"C:\\scdebug.dbg";
      #endif
      cout <<endl;
      cout <<"  -e or --edit filename : Starts in the internal editor with a file loaded" <<endl;
      cout <<"  -h or --help          : Show this Information" <<endl;
      exit(0);
    }

  }
  InitSystem();

  //Uncomment to see sc's idea of commandline parameters
/*  if (IsDebuggingEnabled()) {
    cInfoWindow iw;
    for (int i=0; i<argc; i++) {
      string temp=(MkStr(i)+":");
      temp+=argv[i];
      iw+=temp;
    }
    iw.Show();
  }*/
  

  #ifdef _TARGET_LINUX_
    if (Configuration.rootwarn) {
      if (getenv("USER")) {
        string s=getenv("USER");
        if (s=="root") {
          ErrorBox("WARNING! You are root and can make a terrible mess!");
          ErrorBox("This warning can be disabled in the configuration file...");
        }
      }
    }
  #endif
}


void ShutDownSequence(bool clearscreen/*=true*/) {
  if (clearscreen) ClearScreenBlack();
  DoneConIO();
  cout <<"Console freed..." <<endl;

  DoneKeyboard();
  cout <<"Keyboard freed..." <<endl;

  ReleaseSignalHandlers();
  cout <<"Signal handlers released..." <<endl;

  Configuration.WriteConfigFile();
  cout <<"Configuration written..." <<endl;
  cout <<"Sunshine Commander was running for "<<st.MkStrElapsed() <<endl;

  if (Configuration.gotolastdir) {
    //Do not try to exit into an archive etc.
    if (IsDirDir(_sc_outdir)) {
      #ifdef _TARGETFLAVOR_UNIX_
        string s=getenv("HOME");
        s=(AddSlash(s)+".scoutdir");
        ofstream f(s.c_str());
        if (!f.fail()) {
          f <<_sc_outdir;
          f.close();
        }
      #endif

      #ifdef _TARGETFLAVOR_DOS_
        SetDir(_sc_outdir);
      #endif
    }
  }
}

