#include "fm_textfile.h"
#include "fm_fileinfo.h"
#include "fm_window.h"
#include "fm_conf.h"
#include <fstream.h>


cTextFile::~cTextFile() {
  //  delete f;
}

bool cTextFile::ReadWholeFile(const string& fname, tTextMode tm/*=tmNormal*/) {
  if (!StartReadFromFile(fname,tm)) return false;
  while (ContinueRead()) {;};
  return true;
}

bool cTextFile::StartReadFromFile(const string& fname, tTextMode tm/*=tmNormal*/) {
  mode=tm;
//  if (IsFileBinary(fname)) mode=tmHex;
  f=GetFilePointer(fname); //Should now work with arc,ftp etc.

  return (f->Open(omGZRead)); //We want to expand gz-data
}

bool cTextFile::ContinueRead() {
  switch (mode) {
    case tmHex : return ReadAsHex();
    default : return ReadAsNormal();
  }
}

int cTextFile::GetPercentRead() {
  if (f->Size()!=0) return ((f->Pos()*100)/ f->Size()); else return 0;
}

bool cTextFile::ReadAsHex(int transfsize/*=0*/, unsigned char transferred[]/*=NULL*/) {
  flavor=Binary;
  unsigned char buf[2048];
  static unsigned int offset=0;
  int p=0;
  static string s, chars;
  int read;
  if (transfsize!=0) { //Something is transferred from ReadAsNormal
    for (int i=0; i<transfsize; i++) buf[i]=transferred[i];
    read=transfsize;
  } else {
    read=f->Read(&buf,sizeof(buf));
  }
  
  if (read==0) {

    if (s!="") {
      s+=GetSpaces((16-(p%16))*3);
      lines+=(s+"  "+chars);
    }
    DisposeFilePointer(f);
    return false;
  }
  s=(MkHex(offset)+"   ");
  chars="";

  while (p<read) {
    s+=(MkHex(buf[p])+" ");
    if (buf[p]>31) chars+=buf[p]; else chars+='*';
    p++;
    if (!(p%16)) {
      offset+=16;
      lines+=(s+"  "+chars);
      chars="";
      s=(MkHex(offset)+"   ");
    }
  }
  return true;
}

bool cTextFile::ReadAsNormal() {
  unsigned char buf[2048];
  static string s="";
  int read=f->Read(&buf,sizeof(buf));
  if (read==0) { //finished reading
    f->Close();
    if (s!="") {
      lines+=s;  //add last line even if not terminated
    }
    return false;
  }

  if (!sampletaken) { //Check for binary file
    sampletaken=true;
    int testsize=(read>256 ? 256 : read);
    bool bin=false;
    for(int n=0; n<testsize; n++) {if (buf[n]=='\0') {bin=true;}}
    if (bin) {
      mode=tmHex;
      return ReadAsHex(read, &buf[0]);
    }
  }
  
  int p=0;
  while (p<read) {
    if (buf[p]!=0x0D && buf[p]!=0x0A) { //end of line?
      if (buf[p]>31) s+=(buf[p]); else {
        if (buf[p]==9) { //Tab
          do {
            s+=" "; //fill in spaces as Tab
            tabspresent=true;
          } while ((s.length() % Configuration.tabsize)!=0);
        }
      }
    } else {
      if (buf[p]==0x0D) {
        if (buf[p+1]==0x0A && p+1<read) { //Both FF and CR == DOS-flavor
          if (flavor==Unset) flavor=DOS;
          p++;
        } else if (flavor==Unset) flavor=Mac;
      } else if (flavor==Unset) flavor=Unix;
      lines+=s; s="";
    }
    p++;
  }
  return true;
}

bool cTextFile::WriteToFile(const string& name) {
  if (flavor==Binary) ErrorBox("Ooops, how did you get here? Cannot write binary file!");
  f=GetFilePointer(name);
  if (!f->Open(omWrite)) return false;
  char buf[2048];
  int bp=0;
  for(int i=0; i<lines.Size(); i++) {
    for (int ii=0; ii<lines[i].length(); ii++) {
      buf[bp++]=lines[i][ii];
      if (bp==sizeof(buf)) {
        f->Write(buf,bp);
        bp=0;
      }
    }
    switch (flavor) {
      case Unix : buf[bp++]=0x0A; break;
      case Mac  : buf[bp++]=0x0D; break;
      default : { //DOS
        buf[bp++]=0x0D;
        if (bp==sizeof(buf)) {
          f->Write(buf,bp);
          bp=0;
        }
        buf[bp++]=0x0A;
      }
    }
    if (bp==sizeof(buf)) {
      f->Write(buf,bp);
      bp=0;
    }
  }
  if (bp>0) {
    f->Write(buf,bp);
  }
  DisposeFilePointer(f);
  return true;
}

string& cTextFile::operator [] (long int index) {
  //if (index>-1 && index<lines.Size()) return lines[index];
  return lines[index];
}


void cTextFile::Remove(long int index) {
  bool b=lines.RemoveItem(index);
  int i=lines.Size();
}

long cTextFile::FindString(const string& searchfor,long startline/*=0*/) {
  if (startline>=lines.Size()) return -1;
  for (long l=startline; l<lines.Size(); l++) {
    if (lines[l].find(searchfor)!=-1) return l;
  }
  return -1;
}
