#include "fm_defs.h"
#include "fm_tree.h"
#include "fm_conio.h"
#include "fm_window.h"
#include "fm_fileinfo.h"
#include "fm_entry.h"
#include "fm_helpscreens.h"
#include "fm_conf.h"
#include "fm_debug.h"
#include "fm_scd.h"
#include "fm_sysinfo.h"
#ifdef _TARGETFLAVOR_DOS_
  //#ifdef _TARGET_WIN32_
    #ifndef _TARGET_MINGW_
      #include <sys/vfs.h>
    #endif
    #include <sys/stat.h>
 // #endif
  #include <dos.h>
#endif
#include "sc_shell.h"

cFmTree* FmTree;
string Tree_GotoDestination="";

#ifdef _TARGETFLAVOR_UNIX_
  #define _MOUNTSTOKEN "[mounts]"
#endif

//Construc from direntry
cTreeNode::cTreeNode(const cDirEntry& de, cTreeNode* tn) {
  fullname=de.fullname; name=de.name;

  parent=tn;
  samef=children=NULL;
  packed=de.isarchive;
  if (packed) fullname=("{"+fullname+"}");
  ismount=exp=current=false;
  #ifdef _SYMLINKS_POSSIBLE_
    islink=de.islink; linkto=de.linkto;
  #else
    islink=false; linkto="";
  #endif

}

//Construct from name
cTreeNode::cTreeNode(const string& s,cTreeNode* brother, cTreeNode* mother) {
  fullname=s; name=s;
  packed=exp=current=false;
  parent=mother;
  children=NULL;
  samef=brother;
  ismount=islink=false; linkto="";
}

cTreeNode::~cTreeNode() {
  if (samef) delete samef;       //recursive destruction
  if (children) delete children; //recursive
}


//returns true if it found subdirs
bool cTreeNode::Expand(bool recursive/*=false*/) {
  if (islink) return false;
  if (name==_PRINTERTOKEN) {
    exp==true; children=NULL;
    return false;
  }
  if (!exp && !children) {
    exp=true;
    tDirContainer dirs;
    if (ScanForDirs(dirs,fullname,true,true)) {
      cTreeNode *tt1=NULL, *tt2=NULL;
      for (int i=0; i<dirs.Size(); i++) {
        tt1=new cTreeNode(dirs[i],this);
        if (!children) children=tt1;
        if (tt2) tt2->samef=tt1;
        tt2=tt1;
      }
    }
  }
  //If recursive then expand all children (with recursion enabled)
  if (recursive && children) {
    cTreeNode *tp=children;
    while (tp) {
      if (IsKeyPressed()) {
        if (GetSomeKey()==kESC) return false;
      }
      if (tp->Expand(true)) tp=tp->samef; else return false;
    }
    return true;
  }
  return true;
}
    
void cTreeNode::Contract() {
  if (children) delete children;
  exp=false; children=NULL;
}


cFmTree::cFmTree() {
  #ifdef _FTP_POSSIBLE_
  top=new cTreeNode("/",
        new cTreeNode(_FTPTOKEN,
          new cTreeNode(_PRINTERTOKEN,
            #ifdef _TARGETFLAVOR_UNIX_
              new cTreeNode(_MOUNTSTOKEN,NULL))));
              mounts=top->samef->samef->samef; ///OUCH!!!!!
            #else
              NULL)));
            #endif
  #else
  top=new cTreeNode("/",
        new cTreeNode(_PRINTERTOKEN,NULL));
  #endif

  #ifdef _TARGETFLAVOR_UNIX_
    top->Expand(); //in dos drives are expanded later
  #else
    top->exp=true;
  #endif
}

cFmTree::~cFmTree() {
  delete top;
}

void cFmTree::ClearCurrent(cTreeNode* node) {
  cTreeNode* tn=node;
  while (tn) {
    if (tn->current) tn->current=false;
    if (tn->exp && tn->children) ClearCurrent(tn->children);
    tn=tn->samef;
  }
}

void cFmTree::MakeShowTree(cTreeNode* node, const string& backpack, int level, int& current) {
  if (Configuration.usesimplechars) {
    //show_first=" ,-`";
    show_dir=" |--";
    show_bottom=" ---";
    show_line=" |  ";
  } else {
    //show_first=" ";
    show_dir=" ";
    show_bottom=" ";
    show_line="   ";
  }

  cTreeNode *tp=node;
  string bp,ts;
  while (tp!=NULL) {
    sShowNode sn;
    sn.link=tp;
    sn.fullpath=tp->fullname;
    sn.current=false;
    ts=backpack;
    if (tp->name=="/") {
      ts=(' '+box.upleft+box.hline+box.hline); //never backpacked :-)
      bp=show_line;
    } else {
      if (tp->samef) {
        ts+=show_dir;
        bp=(backpack+show_line);
      } else {
        ts+=show_bottom;
        bp=(backpack+"    ");
      }
    }
    sn.realnamestart=ts.length();

    if (tp->current) {
      current=showlines.Size();
      if (tp->packed)
        sn.showline=(ts+"[{"+tp->name+"}]");
      else
        sn.showline=(ts+'['+tp->name+']');
    } else {
      if (tp->packed)
        sn.showline=(ts+'{'+tp->name+'}');
      else
        if (tp->islink) {
          sn.showline=(ts+"@"+tp->name);
        } else {
          sn.showline=(ts+tp->name);
        }
    }
    if (!tp->islink) {
      if (!tp->exp) sn.showline+=" ...";
    } else {
      sn.showline+=" -> ";
    }

    showlines+=sn;
    if (tp->children) {
      MakeShowTree(tp->children,bp,level+1,current);
    }
    tp=tp->samef;
  }
}

void cFmTree::UpdateView() {
  Wr(_centered,1,title);
  if (allowgotodest) {
    string s=(gotodest ? "Goto destination (F3)" : "Do not goto dest (F3)");
    Wr(ScreenW()-s.length()-2,1,s);
  }
  int lowerline=topline+ScreenH()-3;
  if (lowerline>showlines.Size()) {
    lowerline=showlines.Size();
    topline=showlines.Size()-ScreenH()+3;
    if (topline<0) topline=0;
  }
  for (int i=topline; i<lowerline; i++) {
  
    if (i==cur) {
      int namestart=showlines[i].realnamestart;
      string ts=showlines[i].showline.substr(0,namestart);
      //debugging:
      //char hej[200]; strcpy(hej,ts.c_str());
      Wr(1,i-topline+2,ts,chWHITE,cBLUE,ScreenW());
      ts=showlines[i].showline.substr(namestart,999);
      if (Configuration.monochromesupport && MonochromeConsole()) {
        ts=('>'+ts+'<');
      }
      //strcpy(hej,ts.c_str());
      Wr(namestart+1,i-topline+2,ts,chWHITE,cRED);
      Wr(namestart+1+ts.length(),i-topline+2,"",ScreenW());
    } else {
      Wr(1,i-topline+2,showlines[i].showline,chWHITE,cBLUE,ScreenW());
    }
  }
  if (lowerline<ScreenH()-2) {
    for (int i=lowerline+2; i<ScreenH(); i++) {
      Wr(1,i,GetSpaces(ScreenW()));
    }
  }
  int namestart=showlines[cur].realnamestart;
  string ts=showlines[cur].showline.substr(namestart,999);
  if (showlines[cur].link->islink) {
    ts+=(showlines[cur].link->linkto);
  }

  Wr(1,ScreenH(),ts,cCYAN,cBLACK,ScreenW()-1);
  Wr(1,ScreenH()-1,"",cYELLOW,cBLACK,ScreenW()-1);
  Wr(1,ScreenH()-1,"Search: "+search,cYELLOW,cBLACK);
}

void cFmTree::CalcTopline() {
  topline=(cur-((ScreenH()-3)/2));
  if (topline<0) topline=0;
}

int cFmTree::Update() {
  showlines.Clear();
  int current;
  string temp="";
  #ifdef _TARGETFLAVOR_UNIX_
    CheckMounts();
  #endif
  MakeShowTree(top,temp,0,current);
  //CalcTopline();
  return current;
}

bool cFmTree::HandleDirSearch(tKey k) {
  if (k==kTab) {
    if (search=="") {
      search="/"; cur=0; CalcTopline(); return true;
    } else {
      //Begin searching from current position downwards
      for(int i=cur+1; i<showlines.Size(); i++) {
        if (MatchStart(search,showlines[i].fullpath)) {
          cur=i; search=showlines[i].fullpath+"/";
          if (showlines[i].link->Expand()) {
            Update(); //Recalc showlines
          }
          CalcTopline();
          return true;
        }
      }
      //If not found search from the top to the current position
      for(int i=0; i<cur; i++) {
        if (MatchStart(search,showlines[i].fullpath)) {
          cur=i; search=showlines[i].fullpath+"/";
          if (showlines[i].link->Expand()) {
            Update(); //Recalc showlines
          }
          CalcTopline();
          return true;
        }
      }
    }
  }
  if (k==kBackspace) {
    if (search.length()>0) {
      search=search.substr(0,search.length()-1);
      return true;
    }
  }
  if (KeyIn(k,_KEYSET_LETTERS,kSlash,kBackSlash)) {
    search+=Key_LastASCIIChar();
    return true;
  }
  return false;
}


bool cFmTree::SearchDirChar() {
  //Begin searching from current position downwards
  for(int i=cur+1; i<showlines.Size(); i++) {
    char c=Upcase(Key_LastASCIIChar());
    if (Upcase(showlines[i].showline[showlines[i].realnamestart])==c) {
      cur=i;
      CalcTopline();
      return true;
    }
  }
  //If not found search from the top to the current position
  for(int i=0; i<cur; i++) {
    char c=Upcase(Key_LastASCIIChar());
    if (Upcase(showlines[i].showline[showlines[i].realnamestart])==c) {
      cur=i;
      CalcTopline();
      return true;
    }
  }
  return false; //Not found anywhere
}

bool cFmTree::TreatKey(tKey& k) {
  if (KeyIn(k,_KEYSET_ALTLETTERS) || k==kDot) {
    return SearchDirChar();
  }
  if (KeyIn(k,_KEYSET_NUMBERS)) {
    if (Configuration.quickdirs[Key_LastNumber()]!="") {
      if (DoesDirExist(Configuration.quickdirs[Key_LastNumber()])) {
        cTreeNode* dirlink=FillInDir(Configuration.quickdirs[Key_LastNumber()],false);
        Update();
        cur=FindDir(dirlink);
        CalcTopline();
      }
    }
    return true;
  }
  if (KeyIn(k,_KEYSET_ALTNUMBERS)) {
    int i=Key_LastNumber();
    string s="Set quickdir ";
    s+=MkStr(i);
    s+=" to current dir? (~Y/N)";
    if(KeyIn(PromptBox("Confirmation",s,kEnter,kESC,kY,kN),kEnter,kY)) {
      Configuration.quickdirs[i]=showlines[cur].fullpath;
    }
    Update();
    return true;
  }
  switch (k) {
    //Simple move down
    case kDown : {
      if (cur<showlines.Size()-1) cur++;
      if (cur>(topline+ScreenH()-4)) {
        topline++;
        if (topline>(ScreenH()-3)) CalcTopline();
      }
//      CalcTopline();
      search="";
      return true;
    }

    //Simple mode up
    case kUp : {
      if (cur>0) cur--;
      if (topline>cur) {
        topline--;
        if (topline<0) CalcTopline();
      }
      search="";
      return true;
    }

    //Page Up
    case kPgup : {
      for(int i=0; i<ScreenH()-2; i++) _TreatKey(kUp);
      return true;
    }

    //Page down
    case kPgdn : {
      for(int i=0; i<ScreenH()-2; i++) _TreatKey(kDown);
      return true;
    }

    //Goto top
    case kHome : {cur=0; CalcTopline(); return true;};

    //Goto bottom
    case kEnd : {
      cur=showlines.Size()-1; //CalcTopline();
      topline=showlines.size()-ScreenH()+3;
      if (topline<0) topline=0;
      return true;
    };

    //Goto next sub, expand if necessary
    case kRight : {
      search="";
      if (showlines[cur].link->islink || showlines[cur].link->ismount) {
        if (showlines[cur].link->linkto=="") return false;
        cTreeNode* dirlink=FillInDir(showlines[cur].link->linkto,false);
        Update();
        cur=FindDir(dirlink);
        CalcTopline();
        return true;
      }
      if (showlines[cur].link->Expand()) {
        Update();
        cur++;
        CalcTopline();
      }
      return true;
    }

    //goto Parent if any
    case kLeft : {
      cTreeNode* tn=showlines[cur].link->parent;
      if (tn!=NULL) {
        do {
          _TreatKey(kUp);
        } while (tn!=showlines[cur].link);
      }
      search="";
      return true;
    }

    //Show Dir info
    case kF2 : {
      if (showlines[cur].link->parent==NULL) {
        if (showlines[cur].link->name!="FTP" && showlines[cur].link->name!="Printer") {
          ShowDirInfo(showlines[cur].fullpath);
        }
      } else ShowDirInfo(showlines[cur].fullpath);
      return true;
    }

    //Expand branch fully
    case kCtrlRight :
    case kAltRight :
    case kPlus :
    case kSpace : {
      if (showlines[cur].fullpath!="FTP" && showlines[cur].fullpath!="Printer") {
        ShowMessage("Expanding branch...");
        showlines[cur].link->Expand(true); //do it recursively
        Update();
        return true;
      }
      return false;
    }

    //Contract branch
    case kCtrlLeft :
    case kAltLeft :
    case kMinus : {
      if (showlines[cur].link->parent!=NULL) {
        showlines[cur].link->Contract(); //contract the dir
        Update();
        return true;
      }
      return false;
    }

    //Create a new directory
    case kInsert : {
      #ifdef _FTP_POSSIBLE_
      if (showlines[cur].fullpath==_FTPTOKEN) {
        if (EnterNewFtpSite()) {
          Update();
          return true;
        }
        return false;
      }
      #endif
      static string s="";
      if (InputWindow("New dir:",s,50)) {
        string newdir=(AddSlash(showlines[cur].fullpath)+s);
        if (MkDir(newdir)) {
          showlines[cur].link->Contract(); //refresh subdirs
          showlines[cur].link->Expand();
          Update();
          while (showlines[cur].fullpath!=newdir) _TreatKey(kDown);
        } else {
          ErrorBox("Could not create directory!");
        }
      }
      return true;
    }

    //Delete a directory
    case kDelete : {
      cTreeNode* tn=showlines[cur].link->parent;
      if (tn==NULL) return false;
      if (RmDir(showlines[cur].fullpath)) {
        _TreatKey(kUp);
        tn->Contract(); //refresh subdirs
        tn->Expand();
        Update();
      } else ErrorBox("Could not delete directory! Sure it is empty?");
      return true;
    }


    //Who uses this, anyway?
    /*case kBackspace : {
      string s=SCD_GetDir();
      if (s!="") {
        cTreeNode* dirlink=FillInDir(s,false);
        Update();
        cur=FindDir(dirlink);
        CalcTopline();
        return true;
      }
      return true;
    }*/

    //Show Help
    case kF1 : ShowTreeHelp(); return true;
    case kF5 : Rescan(); return true;
    case kF6 : ShowDriveInfo(); return true;

    case kF3 : if (allowgotodest) gotodest=!gotodest; return true;
    case kEnter : {
      #ifdef _TARGETFLAVOR_UNIX_
      //cannot press enter on the mountsroot
      if (showlines[cur].link->fullname==_MOUNTSTOKEN) k=kDummyKey;
      #endif
    }
    
    default : return HandleDirSearch(k);
  }
}

string cFmTree::GetPath(const string& headline, const string& currentdir,const string& defaultdest, bool _allowgotodest=false, bool _enteredslash=false) {
  allowgotodest=_allowgotodest;
  gotodest=false;
  Tree_GotoDestination="";
  if (_enteredslash) search="/"; else search="";
  title=headline;
  tKey k;
  #ifdef _TARGETFLAVOR_DOS_
    CheckDrives();
  #endif
  ClearCurrent(top);
  if (FillInDir(currentdir)==NULL) return "";
  if (defaultdest!="" && DoesDirExist(defaultdest)) FillInDir(defaultdest);
  string temp="";
  cur=Update();
  /*cur=-1;
  showlines.Clear();
  MakeShowTree(top,temp,0,cur);*/
  CalcTopline();
  ClearScreen();

  bool refresh=true;
  do {
    if (refresh) UpdateView();

    k=GetSomeKey();
    refresh=TreatKey(k);

  } while (!KeyIn(k,kEnter,kESC, kF10));
  if (gotodest) Tree_GotoDestination=showlines[cur].fullpath;
  if (k==kEnter) {
    if (showlines[cur].link->islink || showlines[cur].link->ismount) {
      if (showlines[cur].link->linkto=="") return showlines[cur].fullpath;
      return (showlines[cur].link->linkto);
    } else {
      return showlines[cur].fullpath;
    }
  }
  return "";

};

int cFmTree::FindDir(cTreeNode* searchfor) {
  for (int i=0; i<showlines.Size(); i++) {
    if (showlines[i].link==searchfor) return i;
  }
  return 0;
}

cTreeNode* cFmTree::FillInDir(const string& d, bool settocurrent/*=true*/) {
  //OOOhhh, this is ugly, but it works
  ShowMessage("Building tree...");
  Dbg("fm_tree: inserting "+d);
  if (d=="/") {
    if (settocurrent) top->current=true;
    top->Expand();
    return top;
  }
  cTreeNode* tp=top->children; //subdirs from root
  string s=d;
  if (IsDirArc(s)) {
    int i=s.find('}');
    s.replace(i,1,"/");
    s=s.substr(1,999);
    s=StripSlash(s);
  }
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(s)) {
    //Start from ftproot
    tp=top;
    while (tp->name!=_FTPTOKEN) tp=tp->samef;
    if (s==_FTPTOKEN) {//Stop here
      if (settocurrent) tp->current=true;
      return tp;
    }
    int i=s.find(']');
    s.replace(i,1,"/");
    s=s.substr(1,999);
//    s=StripSlash(s);
    s=StrReplace(s,"//","/");
    tp->Expand();
    tp=tp->children;
  }
  #endif
  if (s[0]=='/' && s.length()>1) {
    s=s.substr(1,999); //in linux drop root slash
  }
  int i;
  string ts;
  do {
    i=s.find('/');
    if (i>-1) {
      ts=s.substr(0,i);
      #ifdef _TARGETFLAVOR_DOS_
        if (ts[1]==':') ts=AddSlash(ts);
      #endif
      s=s.substr(i+1,999);
    } else {
      ts=s;
      s="";
    }

    //Work around because DOS fucks up casing, and does not allow two
    //dirs with same name but diff. case, anyway
    #ifdef _TARGETFLAVOR_DOS_
      while (tp!=NULL && Upcase(tp->name)!=Upcase(ts)) {
    #else
      while (tp!=NULL && tp->name!=ts) {
    #endif
      tp=tp->samef;
    }
    if (tp==NULL) {
      MessageBox("TREE FAILURE!!!!");
      return NULL;
      //halt();
    }
    tp->Expand();
    if (s!="") {
      if (tp->children==NULL) {
        MessageBox("TREE FAILURE!!!!");
        return NULL;
      } else {
        tp=tp->children;
      }
    }
  } while (s!="");
  if (settocurrent) tp->current=true;



  return tp;

}

void cFmTree::Rescan() {
  string n=showlines[cur].fullpath;
  top->Contract();
  #ifdef _TARGETFLAVOR_UNIX_
    top->Expand(); //No drives, just a rootdir
  #else
    CheckDrives();
  #endif
  FillInDir(n);
  cur=Update();
}


#ifdef _TARGETFLAVOR_DOS_
void cFmTree::CheckDrives() {
  cTreeNode *tp=top->children; //first one from root is drives
  #ifndef _TARGET_MINGW_
  if (!tp) { //only executed first time
    cTreeNode *tp2=NULL;

    top->children=new cTreeNode("a:/",NULL); //force floppydisk
    tp2=top->children;
    
    for(char c='c'; c<'z'; c++) {
      struct statfs fs;
      string s; s=c; s+=":/";
      int error=statfs(s.c_str(), &fs);
      if (!error) {
        tp=new cTreeNode(s,NULL);
        tp->parent=top;
        if (tp2) tp2->samef=tp;
        if (!top->children) top->children=tp;
        tp2=tp;
      } else {

      }
    }
  }
  #endif
}
#endif //_TARGETFLAVOR_DOS_

#ifdef _TARGETFLAVOR_UNIX_
void cFmTree::CheckMounts() {
  scDELETE(mounts->children);
  cTreeNode *tp1=NULL,*tp2=NULL;
  tFilesystems filesys;
  GetBriefFilesystemInfo(filesys);
  for (int i=0; i<filesys.size(); i++) {
    if (filesys[i].mountpoint=="/") continue;
    tp1=new cTreeNode(filesys[i].mountpoint,NULL,mounts);
    tp1->ismount=true;
    tp1->linkto=filesys[i].mountpoint;
    if (mounts->children==NULL) mounts->children=tp1;
    if (tp2!=NULL) tp2->samef=tp1;
    tp2=tp1;
    tp1->exp=true;
  }
  mounts->exp=true;
}
#endif _TARGETFLAVOR_UNIX_

