/***************************************************************************
                       fm_vfs.cpp  -  the filesystem reader
                             -------------------
    begin                : Thu Dec 23 1999
    copyright            : (C) 1999 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>
#include <iostream.h>
#include <dirent.h>
#include <sys/stat.h>
#include "target.h"

//#ifndef _TARGET_WIN32_
#ifndef _TARGET_MINGW_
  #include <sys/vfs.h>
#endif

  #include <unistd.h>
//#endif

#include <errno.h>


#ifdef _TARGET_DJGPP_
  #include <dir.h> //for findfirst/next
  #include <dos.h>
  #include <io.h>
#endif

#include "fm_vfs.h"
#include "fm_debug.h"
#include "fm_defs.h"
#include "fm_sort.h"
#include "fm_window.h" //For rotator
#include "array.h"
#include "fm_textfile.h" //For filesystemscan in Linux
#include "sc_archive.h"

#ifdef _FTP_POSSIBLE_
  #include "sc_ftp.h"
#endif

#include "strnatcmp.h"

string cDirEntry::GetFileTimeStr() const {
  tm* ftime=gmtime(&time);
  string s="";
  s+=(Leading0(MkStr(ftime->tm_hour),2)+":");
  s+=(Leading0(MkStr(ftime->tm_min),2)+":");
  s+=(Leading0(MkStr(ftime->tm_sec),2)+" ");
  s+=(Leading0(MkStr(ftime->tm_mday),2)+"/");
  s+=(Leading0(MkStr(ftime->tm_mon),2)+"/");
  int year;
  if (ftime->tm_year>99) year=ftime->tm_year-100; else year=ftime->tm_year;
  s+=(Leading0(MkStr(year),2)+" ");
  return s;
}

string cDirEntry::GetFileModeStr() const {
  string s="";
  if (useattr) {
    #ifdef _TARGETFLAVOR_DOS_  //In linux we should never get here!
    s+=' ';
    if (modeattr & _A_ARCH) s+='A'; else s+=' ';
    if (modeattr & _A_RDONLY) s+='R'; else s+=' ';
    if (modeattr & _A_HIDDEN) s+='H'; else s+=' ';
    if (modeattr & _A_SYSTEM) s+='S'; else s+=' ';
    #endif
  } else {
    s+=' ';
    if (modeattr & S_IRUSR) s+="r"; else s+=" ";
    if (modeattr & S_IWUSR) s+="w"; else s+=" ";
    if (modeattr & S_IXUSR) s+="x"; else s+=" ";

    if (modeattr & S_IRGRP) s+="r"; else s+=" ";
    if (modeattr & S_IWGRP) s+="w"; else s+=" ";
    if (modeattr & S_IXGRP) s+="x"; else s+=" ";

    if (modeattr & S_IROTH) s+="r"; else s+=" ";
    if (modeattr & S_IWOTH) s+="w"; else s+=" ";
    if (modeattr & S_IXOTH) s+="x"; else s+=" ";
  }
  return s;
}

#ifdef _SYMLINKS_EXIST_
void cDirEntry::CheckLink() {
  struct stat s;
  if(!stat(fullname.c_str(), &s)) { //Can it be stat'ed? Target not resolved if not
    char temp[300];
    realpath(fullname.c_str(),temp);
    if (temp!=NULL) linkto=temp; else linkto="";
    isdir=S_ISDIR(s.st_mode);
    badlink=(linkto=="");
  } else {
    badlink=true;
    char buf[300];
    if (readlink(fullname.c_str(),buf,sizeof(buf))!=-1) { //Find target anyway
      linkto=buf;
    } else {
      linkto="";
    }
  }
}
#endif

/*#ifdef _TARGET_UNIX_
void cDirEntry::MakeLink(const string& _name, const string& _fullname) {
  islink=true; CheckLink();
  name=_name; fullname=_name;
  isexecutable=isarchive=isdir=istagged=iscolltagged=false;
}
#endif
*/

#ifdef _FTP_POSSIBLE_
void cDirEntry::MakeFtpSite(const string& _name, const string& _fullname="") {
  isftpsite=true;
  isexecutable=isarchive=tagged=colltagged=false;
  isdir=false;
  if (_name==_FTPTOKEN) {
    name=_name.substr(1,_name.length()-2);
  } else {
    name=_name;
  }
  if (_fullname!="") fullname=_fullname; else fullname=_name;
  isspecial=false; taggable=false;
  #ifdef _SYMLINKS_EXIST_
    islink=false;
  #endif
  modeattr=0;
  size=0;
}
#endif

void cDirEntry::MakeSpecial(const string& _name, const string& _fullname/*=""*/,bool _isdir/*=true*/) {
  isexecutable=isarchive=isftpsite=tagged=colltagged=false;
  isdir=_isdir;
  name=_name; if (_fullname!="") fullname=_fullname; else fullname=_name;
  isspecial=true; taggable=false;
  #ifdef _SYMLINKS_EXIST_
    islink=false;
  #endif
  modeattr=0;
  size=0;
}

/*void cDirEntry::MakeDir(const string& _name,
               const string& _fullname,
               tModeAttr _tma,
               int _modeattr,
               time_t _time) {
  isexecutable=isarchive=istagged=iscolltagged=false;
  isdir=true;

}
*/

void cDirEntry::MakeNormal(const string& _name,
               const string& _fullname,
               tModeAttr _tma,
               int _modeattr,
               time_t _time,
               long int _size) {
  isftpsite=isspecial=tagged=colltagged=false;
  name=_name; fullname=_fullname; time=_time; size=_size;

  modeattr=_modeattr; taggable=true;
  useattr=(_tma==useAttr);
  if (useattr) {
    isdir=((modeattr & 0x10)==0x10);
  } else {
    isdir=S_ISDIR(modeattr);
  }

  //Handle executability
  #ifdef _TARGETFLAVOR_UNIX_
    isexecutable=(modeattr&S_IXUSR || modeattr&S_IXGRP || modeattr&S_IXOTH);
  #endif

  #ifdef _SYMLINKS_EXIST_
    if (islink=(S_ISLNK(modeattr))) {
      CheckLink();
    }
  #endif
  #ifdef _TARGETFLAVOR_DOS_
    string s=GetPartExt(name);
    isexecutable=(s==".EXE" || s==".BAT" || s==".COM" || s==".BTM");// return true;
    if (isdir) size=0;
  #endif

  //Handle archive
  isarchive=false;
  int i=0;
  while (_arc_extensions[i]!="") {//for (int i=0; i<_noarc_extensions; i++) {
    if (MatchEnd(_arc_extensions[i],fullname)) {
      isarchive=true;
      break;;
    }
    i++;
  }
}

#ifdef _TARGET_DJGPP_
time_t MakeTimeFromFF(const ffblk& f) {
  //via tm-struct!  mktime in time.h!
  time_t result;
  tm via;
  via.tm_hour=(f.ff_ftime >> 11) & 0x1f;
  via.tm_min=(f.ff_ftime >>  5) & 0x3f;
  via.tm_sec=(f.ff_ftime & 0x1f) * 2;
  via.tm_mon=(f.ff_fdate >>  5) & 0x0f;
  via.tm_mday=(f.ff_fdate & 0x1f);
  via.tm_year=((f.ff_fdate >> 9) & 0x7f)+80; //years since 1900
  via.tm_yday=via.tm_wday=via.tm_isdst=via.__tm_gmtoff=0; via.__tm_zone=NULL;
  result=mktime(&via);
  return result;
}
#endif

string CurrentDir() {
  char temp[300];
  getcwd(temp,300);
  string s=temp;
  return s;
}

bool SetDir(const string& dirname) {
  return(chdir(dirname.c_str())==0);
}




int CountDir(const string& dir/*="."*/) {
  int count=0;
  DIR *d = opendir(dir.c_str());
  if (d==NULL || readdir(d)==NULL) return -1;
  while (readdir(d)) count++;
  return count;  
}



bool NormalScan(tDirContainer& cont, const string& dir=".",bool onlydirs=false, bool allowarcs=false) {
  string cdir;
  if (dir==".") cdir=AddSlash(CurrentDir()); else cdir=AddSlash(dir);

#ifdef _TARGETFS_STAT_
  Dbg("fm_vfs: trying to open dir: "+cdir);
  DIR *d = opendir(cdir.c_str());
  if (d==NULL || readdir(d)==NULL) { //first entry=".", uninteresting
    Dbg("fm_vfs: opendir failed! ");
    return false;
  }
  cont.Clear();
  Dbg("fm_vfs: opendir succeeded!");
  
  struct dirent *de;
  while (de=readdir(d)) {
    ShowRotator();
    struct stat s;
    string ts=(cdir+de->d_name);
    cDirEntry tde;
    if (!lstat(ts.c_str(), &s)) { //pointer til s
      string name=de->d_name;
      #ifdef _TARGET_CYGWIN_
        tde.MakeNormal(name,cdir+name,useAttr,s.st_mode,s.st_atime,s.st_size);
      #else
        tde.MakeNormal(name,cdir+name,useMode,s.st_mode,s.st_atime,s.st_size);
      #endif
      if (name=="..") tde.taggable=false;
      if (onlydirs) {
        if (tde.isdir && tde.name!="..") {
          cont+=tde;
        } else {
          if (allowarcs && tde.isarchive) {
            cont+=tde;
          }
        }
      } else cont+=tde;
    } else {
      string s=de->d_name;
      Dbg("fm_vfs: failed stat on "+s);
    }
  }
  Dbg("fm_vfs: Closing dir");
  closedir(d);
  Dbg("fm_vfs: Done");
#endif //_TARGETFS_STAT_

#ifdef _TARGETFS_FINDFIRST_
  Dbg("fm_vfs: trying findfirst ->"+cdir);

  #ifdef _TARGET_DJGPP_
  struct ffblk f;
  string s=AddSlash(cdir)+"*";
  int done = findfirst(s.c_str(), &f, 0x10/*0x3f*/); //any file
  if (done) {
    Dbg("fmvfs: findfirst failed");
    return false;
  }
  if (strcmp(f.ff_name,".")==0) done=findnext(&f); //always skip the '.'-entry, if present
  cont.Clear();
  cDirEntry tde;
  while (!done) {
    ShowRotator();
    string name=f.ff_name;
    tde.MakeNormal(name,cdir+name,useAttr,f.ff_attrib,MakeTimeFromFF(f),f.ff_fsize);
    if (name=="..") tde.taggable=false;
    if (onlydirs) {
      if (tde.isdir && name!="..") {
        cont+=tde;
      } else {
        if (allowarcs && tde.isarchive) {
          cont+=tde;
        }
      }
    } else cont+=tde;
    done=findnext(&f);
  }
  #endif
  #ifdef _TARGET_MINGW_
/*  struct _finddata_t
{
        unsigned        attrib;
        time_t          time_create;
        time_t          time_access;
        time_t          time_write;
        _fsize_t        size;
        char            name[FILENAME_MAX];
};*/

  _finddata_t f;
  string s=AddSlash(cdir)+"*.*";
  int handle = _findfirst (s.c_str(), &f);
  if (handle==0) {
    Dbg("fmvfs: findfirst failed");
    return false;
  }
  if (strcmp(f.name,".")==0) handle=_findnext(handle,&f); //always skip the '.'-entry, if present
  cont.Clear();
  cDirEntry tde;
  while (!handle) {
    ShowRotator();
    string name=f.name;
    tde.MakeNormal(name,cdir+name,useAttr,f.attrib,f.time_write,f.size);
    if (name=="..") tde.taggable=false;
    if (onlydirs) {
      if (tde.isdir && name!="..") {
        cont+=tde;
      } else {
        if (allowarcs && tde.isarchive) {
          cont+=tde;
        }
      }
    } else cont+=tde;
    handle=_findnext(handle, &f);
  }
  #endif
#endif //_TARGETFS_FINDFIRST_

  return true;
}


bool ScanForFiles(tDirContainer& cont, const string& dir/*="."*/, bool sort) {
  if (IsDirArc(dir)) {
    if (!ArchiveScan(cont,dir,false)) return false;
  } else {
    #ifdef _FTP_POSSIBLE_
    if (IsDirFtp(dir)) {
      if (!FtpScan(cont,dir,false)) return false;
    } else {
    #endif
      if (!NormalScan(cont,dir,false)) return false;
    #ifdef _FTP_POSSIBLE_
    }
    #endif
  }
  if (sort) {
    Dbg("fm_vfs: Sorting files");
    SortFiles(cont);
    Dbg("fm_vfs: Done");
  }
  return true;
}

bool ScanForDirs(tDirContainer& cont, const string& dir/*="."*/, bool sort/*=true*/, bool allowarcs/*=false*/) {
  if (IsDirArc(dir)) {
    if (!ArchiveScan(cont,dir,true)) return false;
  } else {
    #ifdef _FTP_POSSIBLE_
    if (IsDirFtp(dir)) {
      if (!FtpScan(cont,dir,true)) return false;
    } else {
    #endif
      if (!NormalScan(cont,dir,true,allowarcs)) return false;
    #ifdef _FTP_POSSIBLE_
    }
    #endif
  }
  if (sort) {
    Dbg("fm_vfs: Sorting dirs");
    SortFiles(cont);
    Dbg("fm_vfs: Done");
  }
  return (cont.Size()>0);
}

int CompareName(const cDirEntry& f1, const cDirEntry& f2) {
  return strnatcasecmp(f1.name.c_str(),f2.name.c_str());
  //return strcasecmp(f1.name.c_str(),f2.name.c_str());
}

int Compare(const cDirEntry& f1, const cDirEntry& f2) {
  if (f1.isdir) {
    if (f2.isdir) {
      return CompareName(f1,f2);
    } else {
      return -1;
    }
  }
  if (f2.isdir) return 1;

  if (f1.isarchive) {
    if (f2.isarchive) {
      return CompareName(f1,f2);
    } else {
      return -1;
    }
  }
  if (f2.isarchive) return 1;

  return CompareName(f1,f2);
}


bool operator < (const cDirEntry& f1, const cDirEntry& f2) {
  return (Compare(f1,f2)<0);
}

bool operator == (const cDirEntry& f1, const cDirEntry& f2) {
  return (Compare(f1,f2)==0);
}

/***************************************************************************/


bool GetBriefFilesystemInfo(tFilesystems& cont) {
  cont.Clear();
#ifdef _TARGETFLAVOR_DOS_
  #ifndef _TARGET_MINGW_
  struct statfs fs;
  struct sFileSystem sfs;
  sfs.mountpoint="a:"; sfs.device="a:"; //always include floppy, but do not scan
  sfs.total=sfs.avail=0;
  cont+=sfs;
  for(char c='c'; c<='z'; c++) {
    ShowMessage("Scanning drive "+MkStr(c)+":");
    string s; s+=c; s+=":";
    if (!statfs(s.c_str(), &fs)) {
      sfs.total=fs.f_blocks*fs.f_bsize;
      sfs.avail=(fs.f_bfree*fs.f_bsize);
      sfs.device=c; sfs.device+=":";
      sfs.mountpoint=c; sfs.mountpoint+=":";
      sfs.filesystem="fat";
      cont+=sfs;
    }
  }
  return (cont.Size()>0);
  #endif
#endif //_TARGETFLAVOR_DOS_

#ifdef _TARGETFLAVOR_UNIX_
  cTextFile tf;
  if (tf.StartReadFromFile("/proc/mounts")) {
    while (tf.ContinueRead()) {;};
    for (int i=0; i<tf.Size(); i++) {
      if (tf[i][0]=='#' || tf[i][0]==';') continue;
      struct sFileSystem sfs;
      int p=tf[i].find(' ');
      if (p>-1) {
        sfs.device=tf[i].substr(0,p);
        tf[i]=tf[i].substr(p+1,255);
        p=tf[i].find(' ');
        if (p>-1) {
          sfs.mountpoint=tf[i].substr(0,p);
          tf[i]=tf[i].substr(p+1,255);
          p=tf[i].find(' ');
          if (p>-1) {
            sfs.filesystem=tf[i].substr(0,p);
            //struct statfs fs;
            //string s=sfs.mountpoint+"/.";
            /*if (!statfs(s.c_str(), &fs)) {
              int blocks=fs.f_blocks, bsize=fs.f_bsize, bfree=fs.f_bavail;
              sfs.total=(blocks*bsize);
              sfs.avail=(bfree*bsize);
              cont+=sfs;
            }*/
            if (sfs.mountpoint=="<supermount>") continue;
            cont+=sfs;            
          }
        }
      }
    }
  } else return false;
  return (cont.Size()>0);
#endif //_TARGETFLAVOR_UNIX_
}


/***************************************************************************/
bool GetFullFilesystemInfo(tFilesystems& cont) {
  cont.Clear();
#ifdef _TARGETFLAVOR_DOS_
  #ifndef _TARGET_MINGW_
  struct statfs fs;
  struct sFileSystem sfs;
  sfs.mountpoint="a:"; sfs.device="a:"; //always include floppy, but do not scan
  sfs.total=sfs.avail=0;
  cont+=sfs;
  for(char c='c'; c<='z'; c++) {
    ShowMessage("Scanning drive "+MkStr(c)+":");
    string s; s+=c; s+=":";
    if (!statfs(s.c_str(), &fs)) {
      sfs.total=fs.f_blocks*fs.f_bsize;
      sfs.avail=(fs.f_bfree*fs.f_bsize);
      sfs.device=c; sfs.device+=":";
      sfs.mountpoint=c; sfs.mountpoint+=":";
      sfs.filesystem="fat";
      cont+=sfs;
    }
  }
  return (cont.Size()>0);
  #endif
#endif //_TARGETFLAVOR_DOS_

#ifdef _TARGETFLAVOR_UNIX_
  cTextFile tf;
  if (tf.StartReadFromFile("/proc/mounts")) {
    while (tf.ContinueRead()) {;};
    for (int i=0; i<tf.Size(); i++) {
      struct sFileSystem sfs;
      int p=tf[i].find(' ');
      if (p>-1) {
        sfs.device=tf[i].substr(0,p);
        tf[i]=tf[i].substr(p+1,255);
        p=tf[i].find(' ');
        if (p>-1) {
          sfs.mountpoint=tf[i].substr(0,p);
          tf[i]=tf[i].substr(p+1,255);
          p=tf[i].find(' ');
          if (p>-1) {
            sfs.filesystem=tf[i].substr(0,p);
            struct statfs fs;
            string s=sfs.mountpoint+"/.";
            if (!statfs(s.c_str(), &fs)) {
              int blocks=fs.f_blocks, bsize=fs.f_bsize, bfree=fs.f_bavail;
              /*double d=fs.f_blocks*fs.f_bsize;
              sfs.total=long(d);
              d=(fs.f_bfree*fs.f_bsize);
              sfs.avail=long(d);*/
              sfs.total=(blocks*bsize);
              sfs.avail=(bfree*bsize);
              cont+=sfs;
            }
          }
        }
      }
    }
  } else return false;
  return (cont.Size()>0);
#endif //_TARGETFLAVOR_UNIX_
}

long int GetFileSize(const string& fname) {
  struct stat s;
  if (!stat(fname.c_str(), &s)) {
    return s.st_size;
  } else {
    return -1;
  }

}

time_t GetFileTime(const string& fname) {
  struct stat s;
  if (!stat(fname.c_str(), &s)) {
    return s.st_atime;
  } else {
    return 0;
  }
}

bool DoesDirExist(const string& fname) {
  if (IsDirArc(fname)) {
    return ArcDoesDirExist(fname);
  }
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(fname)) {
    if (fname==_FTPTOKEN) return true;
    return FtpDoesDirExist(fname);
  }
  #endif
  struct stat s;
  if (!stat(fname.c_str(), &s)) {
    return (S_ISDIR(s.st_mode));
  } else {
    return false;
  }
}

bool DoesFileExist(const string& fname) {
  if (IsDirArc(fname)) {
    return ArcDoesFileExist(fname);
  }
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(fname)) {
    return FtpDoesFileExist(fname);
  }
  #endif
  struct stat s;
  if (!stat(fname.c_str(), &s)) {
    return (!S_ISDIR(s.st_mode));
  } else {
    return false;
  }
}

bool DoesExist(const string& name) {
  return (DoesDirExist(name) || DoesFileExist(name));
}

bool MakeComplexDir(const string& dirname, bool thisisafile/*=false*/) {
  string name=dirname;
  if (thisisafile) {
    //remove filename
    int i=name.rfind('/');
    if (i!=-1) {
      name=name.substr(0,i+1); //removing filename
    }
  }  
  for(int i=0; i<name.length(); i++) {
    if (name[i]=='/') {
      string s=name.substr(0,i);
      if (s=="") continue;
      if (!MkDir(s.c_str())) {
        //ignore "already exist"-error, fail on other errors
        if (errno!=EEXIST) return false;
      }
    }
  }
  return true;
}


bool MkDir(const string& dirname) {
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(dirname)) {
    return (FtpMkDir(dirname));
  }
  #endif
  if (IsDirArc(dirname)) {
    return false;
    //return (tpMkDir(dirname));
  }
  #ifdef _TARGET_MINGW_
    return(mkdir(dirname.c_str())==0);
  #else
    return(mkdir(dirname.c_str(),_ALLPERMS)==0);
  #endif
}


bool RmDir(const string& dirname) {
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(dirname)) {
    return (FtpRmDir(dirname));
  }
  #endif
  if (IsDirArc(dirname)) {
    return false;
    //return (tpMkDir(dirname));
  }
  return(rmdir(dirname.c_str())==0);
}

bool DeleteFile(const string& name) {
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(name)) {
    return FtpDeleteFile(name);
  }
  #endif
  if (IsDirArc(name)) {
    return ArcDeleteFile(name);
  }
  return (remove(name.c_str())==0);
}

bool Rename(const string& n1, const string& n2) {
  #ifdef _FTP_POSSIBLE_
  if (IsDirFtp(n1)) {
    return (FtpRename(n1,n2));
  }
  #endif
  if (IsDirArc(n1)) {
    return false;
    //return (FtpRename(n1,n2));
  }
  return (rename(n1.c_str(),n2.c_str())==0);
}

/***************************************************************************/


#ifdef _TARGETFLAVOR_DOS_
sAttrChanges TranslateAttr(int attr) {
  sAttrChanges at;
  at.readonly=(attr & _A_RDONLY)!=0;
  at.hidden = (attr & _A_HIDDEN)!=0;
  at.system = (attr & _A_SYSTEM)!=0;
  at.archive= (attr & _A_ARCH)!=0;
  return at;
}

int TranslateAttr(sAttrChanges at) {
  int i=0;
  if (at.readonly) i=(i | _A_RDONLY);
  if (at.hidden)   i=(i | _A_HIDDEN);
  if (at.system)   i=(i | _A_SYSTEM);
  if (at.archive)  i=(i | _A_ARCH);
  return i;
}
#endif

sModeChanges TranslateMode(int mode) {
  sModeChanges md;
  md.row=(mode & S_IRUSR)!=0; md.wow=(mode & S_IWUSR)!=0; md.xow=(mode & S_IXUSR)!=0;
  md.rgr=(mode & S_IRGRP)!=0; md.wgr=(mode & S_IWGRP)!=0; md.xgr=(mode & S_IXGRP)!=0;
  md.rot=(mode & S_IROTH)!=0; md.wot=(mode & S_IWOTH)!=0; md.xot=(mode & S_IXOTH)!=0;
  md.suid=(mode & S_ISUID)!=0;
  md.sgid=(mode & S_ISGID)!=0;
  #ifdef _TARGET_LINUX
    md.sticky=(mode & S_ISVTX)!=0;
  #endif
  return md;
}

int TranslateMode(sModeChanges md) {
  int i=0;
  if (md.row==_set) i=(i | S_IRUSR);
  if (md.wow==_set) i=(i | S_IWUSR);
  if (md.xow==_set) i=(i | S_IXUSR);

  if (md.rgr==_set) i=(i | S_IRGRP);
  if (md.wgr==_set) i=(i | S_IWGRP);
  if (md.xgr==_set) i=(i | S_IXGRP);

  if (md.rot==_set) i=(i | S_IROTH);
  if (md.wot==_set) i=(i | S_IWOTH);
  if (md.xot==_set) i=(i | S_IXOTH);

  if (md.suid==_set) i=(i | S_ISUID);
  if (md.sgid==_set) i=(i | S_ISGID);
  #ifdef _TARGET_LINUX_
    if (md.sticky==_set) i=(i | S_ISVTX);
  #endif
  return i;
}

#ifdef _TARGETFLAVOR_DOS_
bool GetFileAttr(const string& name, sAttrChanges& at) {
  #ifdef _TARGET_DJGPP_
    int it=_chmod(name.c_str(),0);
  #else
    int it=chmod(name.c_str(),0);
  #endif
  if (it==-1) return false;
  at=TranslateAttr(it);
  return true;
}
#endif

bool GetFileMode(const string& name, sModeChanges& md) {
  struct stat s;
  if (stat(name.c_str(), &s)==0) {
    md=TranslateMode(s.st_mode);
    return true;
  } else return false;
}

#ifdef _TARGETFLAVOR_DOS_
bool SetFileAttr(const string& name, const sAttrChanges& at) {
  #ifdef _TARGET_DJGPP_
    return (_chmod(name.c_str(),1,TranslateAttr(at))!=-1);
  #else
    return (chmod(name.c_str(),TranslateAttr(at))!=-1);
  #endif
}
#endif

bool SetFileMode(const string&name, const sModeChanges& md) {
  return (chmod(name.c_str(),TranslateMode(md))==0);
}




bool IsDirFtp(const string& s) {
  return (s[0]=='[');
}

bool IsDirArc(const string& s) {
  return (s[0]=='{');
}

bool IsDirDir(const string& s) {
  #ifdef _TARGETFLAVOR_DOS_
    return (s[1]==':');
  #else
    return (s[0]=='/');
  #endif
}

string GetTempFileName() {
  char c[1024];
  string temp=tmpnam(c);
  return temp;
}

bool FSysUsesMode(const string& somefullpath) {
  if (IsDirFtp(somefullpath)) return true;
  //should we distinguish between ZIP-archive (attr) and tar.gz (mode)?
  #ifdef _TARGETFLAVOR_UNIX_
    return true;
  #else
    return false;
  #endif
}

/*
struct sParsedDir {
  string completedir;
  string completename;
  tDirType type;
  string dirname;
  string filename;
  string fullname;
  string fileext;
  string arc_dirname;
  string arc_filename;
  string arc_fullname;
};*/

sParsedDir PleaseParseDir(const string& name, bool forcedir/*=false*/) {
  sParsedDir pd;
  pd.completename=name;
  if (name[0]=='{' || name[0]=='[') {
    int pos;
    if (name[0]=='{') {
      pos=name.rfind('}');
      pd.type=dtArc;
    }
    if (name[0]=='[') {
      pos=name.rfind(']');
      pd.type=dtFtp;
    }
    if (pos!=-1) {
      pd.fullname=name.substr(1,pos-1);
      pd.arc_fullname=name.substr(pos+1,999);
      if (forcedir) {
        pd.arc_fullname=AddSlash(pd.arc_fullname);
        pd.arc_dirname=GetPartDir(pd.arc_fullname);
        pd.arc_filename="";
        pd.arc_fileext="";
      } else {
        pd.arc_dirname=GetPartDir(pd.arc_fullname);
        //if (forcedir && pd.arc_dirname=="" && pd.arc_fullname!="") pd.arc_dirname=pd.arc_fullname;
        pd.arc_filename=GetPartName(pd.arc_fullname);
        pd.arc_fileext=GetPartExt(pd.arc_fullname);
      }
    } else exit(666);
  } else {
    pd.type=dtDir;
    pd.fullname=name;
    pd.arc_dirname=pd.arc_filename=pd.arc_fullname=pd.arc_fileext="";
  }
  pd.dirname=GetPartDir(pd.fullname);
  pd.filename=GetPartName(pd.fullname);
  pd.fileext=GetPartExt(pd.fullname);
  return pd;
}



