/***************************************************************************
                          fm_vfs.h  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _FM_VFS
#define _FM_VFS

#include <sys/stat.h>
#include "fm_str.h"
#include "array.h"
#include "target.h"
#ifdef _TARGET_DOS_
  #include <dos.h>
#endif

#ifdef _TARGET_CYGWIN_
  #define _A_ARCH    0x20
  #define _A_RDONLY  0x01
  #define _A_HIDDEN  0x02
  #define _A_SYSTEM  0x04
/*  */
#endif

#ifdef _NO_STATFS_
#include <dos.h>
struct statfs {
    long        f_type;
    long        f_bsize;
    long        f_blocks;
    long        f_bfree;
    long        f_bavail;
    long        f_files;
    long        f_ffree;
    //fsid_t      f_fsid;
    long        f_magic;
  };

  int statfs(const char *dir, /*struct*/ statfs * statstruct) {
    struct _diskfree_t df;
    _getdiskfree(0,&df);
    statstruct->f_bsize=df.sectors_per_cluster*df.bytes_per_sector;
    statstruct->f_blocks=df.total_clusters;
    statstruct->f_bfree=statstruct->f_bavail=df.avail_clusters;
    return 0;
}
#endif //_NO_STATFS_

enum tModeAttr {useMode, useAttr};
enum tDirType {dtDir, dtArc, dtFtp};

struct sParsedDir {
  string completedir;
  string completename;
  tDirType type;
  string dirname;
  string filename;
  string fullname;
  string fileext;

  string arc_dirname, arc_filename, arc_fullname, arc_fileext;
};

sParsedDir PleaseParseDir(const string&,bool=false);

bool IsDirFtp(const string& s);
bool IsDirArc(const string& s);
bool IsDirDir(const string& s);


class cDirEntry {
public:
  string name;
  string fullname;

  #ifdef _SYMLINKS_EXIST_
    string linkto;
    bool islink;
    bool badlink;
    void CheckLink();
  #endif

  int modeattr;
  bool useattr;

  time_t time;

  long int size;

  bool tagged;
  bool taggable;
  bool isspecial;
  bool colltagged;
  bool isdir;
  bool isarchive;
  bool isftpsite;
  bool isexecutable;

  //Makes an untaggable entry with special tag on
  void MakeSpecial(const string& _name,const string& fullname="",bool _isdir=true);

  //Makes an ftpsite without special tag but untaggable
  void MakeFtpSite(const string& _name, const string& _fullname="");

  //Makes a normal entry
  void MakeNormal(const string& _name,
               const string& _fullname,
               tModeAttr _tma,
               int _modeattr,
               time_t _time,
               long int _size);

  string GetFileTimeStr() const;
  string GetFileModeStr() const;

  //Compare friend function
  friend bool operator < (const cDirEntry&, const cDirEntry&);
  friend bool operator == (const cDirEntry&, const cDirEntry&);
};

typedef cArray<cDirEntry> tDirContainer;

//Return true if a proper filelist could be put into the tDirContainer
//The container is untouched if filesearch failed
//Can be sorted or unsorted
bool ScanForFiles(tDirContainer&, const string& dir=".", bool=true);

//Return true if a proper dirlist could be put into the tDirContainer
//The container is untouched if filesearch failed
//Can be sorted or unsorted
bool ScanForDirs(tDirContainer& cont, const string& dir=".", bool sort=true, bool allowarcs=false);

struct sFileSystem {
  string device;
  string mountpoint;
  string filesystem;
  unsigned long int avail;
  unsigned long int total;
};

typedef cArray<sFileSystem> tFilesystems;


//Returns a list of filesysteminfo
//Dos: Available drives
//Linux: mounts read from /proc/mounts
bool GetBriefFilesystemInfo(tFilesystems&);//Just returns names
bool GetFullFilesystemInfo(tFilesystems&); //Scans for space

//Returns the size of a given file, -1 if unsuccessful
long int GetFileSize(const string&);

//Returns the unix-style timestamp of a file
time_t GetFileTime(const string&);

/********************************************/


const int _set=1;
const int _clear=0;
const int _nochange=2;

struct sModeChanges {
  char row,wow,xow;
  char rgr,wgr,xgr;
  char rot,wot,xot;
  char suid,sgid;
  char sticky;
};

struct sAttrChanges {
  char system,hidden,readonly,archive;
};

#ifdef _TARGETFLAVOR_DOS_
  sAttrChanges TranslateAttr(int);
#endif

sModeChanges TranslateMode(int);

#ifdef _TARGETFLAVOR_DOS_
bool GetFileAttr(const string&, sAttrChanges&);
#endif

bool GetFileMode(const string&, sModeChanges&);

#ifdef _TARGETFLAVOR_DOS_
  bool SetFileAttr(const string&, const sAttrChanges&);
#endif

bool SetFileMode(const string&, const sModeChanges&);

/***********************************************/

bool FSysUsesMode(const string& somefullpath);

bool DoesFileExist(const string&); //works only on files
bool DoesDirExist(const string&);  //works only on dirs
bool DoesExist(const string& name);//works both on files and dirs

bool DeleteFile(const string&);

//returns the number of files in a given dir, -1 if unsuccessful
int CountDir(const string& dir=".");

//return the currentdir
string CurrentDir();


//Renames a file
bool Rename(const string&, const string&);

//set the current dir, returns true if successful
bool SetDir(const string&);

//Makes a directory, returns true if successful
bool MkDir(const string&);
bool MakeComplexDir(const string&, bool thisisafile=false);

//Removes a directory, returns true if successful
bool RmDir(const string&);

string GetTempFileName();

#endif //_FM_VFS


