/**************************************************************************
                          fm_viewcon.cpp  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fm_viewcon.h"
#include "fm_conio.h"
#include "fm_str.h"
#include "fm_helpscreens.h"
#include "fm_window.h"
#include "fm_entry.h"
#include "fm_conf.h"
#include "sc_syntax.h"

#define BOTTOMLINE topline+ScreenH()-2
#define LINESONSCREEN ScreenH()-2

cViewWindow::cViewWindow(const string& fname) {
  error=true;
  if (fname=="") {
    ErrorBox("Not a regular file!");
    return;
  }
  if (!lines.StartReadFromFile(fname)) {
    ErrorBox("Could not open file!");
    return;
  }
  error=false;
  finishedreading=false;
  filename=fname;
  topline=0;
  fromleft=0;
  do {
    //read until we can fill a screen
    finishedreading=!lines.ContinueRead();
  } while (lines.Size()<ScreenH() && (!finishedreading));

  if ((CanDoSyntaxByName(fname) || CanDoSyntaxByFirstLine(lines[0]))
       && lines.GetFlavor()!=Binary) {
    while (!finishedreading) {
      finishedreading=!lines.ContinueRead();
    };
    syntax=DoSyntaxHighlighting(fname, lines);
  } else {
    syntax=false;
  }
  Update();
}

cViewWindow::~cViewWindow() {

}

void cViewWindow::Idle() {
  if (!finishedreading) {
    //If reading is not finished, continue
    finishedreading=!lines.ContinueRead();
    WriteHeader();
  } else {
    if (Configuration.clockincorner) ShowTime();
  }
}

void cViewWindow::DoWordWrap() {
  if (finishedreading) {
    if (syntax) return; //Unforseeable consequences ;-) Got the halflife hint?

    //This will be cleaned up some day...
    tStringArray nlines;
    for (int i=0; i<lines.Size(); i++) {
      //ShowProgressWindow(i,lines.Size(),40,"WordWrapping...");
      string s=lines[i];
      if (s.length()<=ScreenW()) { //Does this line fit on screen?
        nlines+=s; //If so just copy it
      } else {
        while (s.length()>ScreenW()) {
          string s2=s.substr(0,ScreenW()); //Cut what fits on screen
          int lastspace=s2.rfind(' '); //Search for a place to wrap
          if (lastspace<5) { //No suitable place found
            s.erase(0,ScreenW()); //So cut it ugly
          } else {
            s.erase(0,lastspace+1); //Else cut it nicely
            s2=s2.substr(0,lastspace); //and adjust the part cutted
          }
          nlines+=s2;
        }
        if (s!="") nlines+=s;  //add the rest if any left
      }
    }
    lines=nlines; fromleft=0;
  }
}

void cViewWindow::WriteHeader() {
  string s=GetTitle();
  if (!finishedreading) {
    s+=(" ("+MkStr(lines.GetPercentRead())+"% read)");
  }
  Wr(3,1,s,chYELLOW,cBLUE,ScreenW()-2); //minus 2 for consolenumber
  s=("Line "+MkStr(topline+1)+"-");
  if (lines.Size()>BOTTOMLINE) {
    s+=MkStr(BOTTOMLINE); //more lines than on screen
  } else {
    s+=MkStr(lines.Size()); //end of file shown
  }
  s+="/"+MkStr(lines.Size())+" Fromleft: "+MkStr(fromleft)+"  Type: ";
  switch (lines.GetFlavor()) {
    case DOS : s+="DOS"; break;
    case Unix : s+="Unix"; break;
    case Mac : s+="Mac"; break;
    case Binary : s+="Binary"; break;
  }
  s+=" (w to wordwrap)";
//  Wr(1,ScreenH(),GetSpaces(ScreenW()-1),chYELLOW,cBLUE);
  Wr(1,ScreenH(),s,chYELLOW,cBLUE,ScreenW()-1);
}

void cViewWindow::Update() {
  WriteHeader();
  for(int i=topline; i<BOTTOMLINE; i++) {
    if ((i-topline)>(lines.Size()-1)) {
      Wr(1,(i-topline)+2,"",cBLACK,cBLUE,ScreenW());
    } else {
      string s;
      if (lines[i].length()>fromleft) {
        //highlight findstring occurences!

        if (syntax) {
          //Transfer whole string to syntax for coloring purposes
          WrSyntax((i-topline)+2,lines[i],fromleft,ScreenW());
        } else {
          s=lines[i].substr(fromleft,ScreenW());
          Wr(1,(i-topline)+2,s,cBLACK,cWHITE,ScreenW());
        }
        if (s.find(findstring)!=-1) {
          //findstring found
        }
      } else {
        Wr(1,(i-topline)+2,"",cBLACK,cWHITE,ScreenW());

      }

    }
  }
}

void cViewWindow::TreatKey(tKey k) {
  bool refresh=true;
  switch (k) {
    case kBlindDown : {
      if (lines.Size()>BOTTOMLINE) topline++;
      return;
    }
    case kBlindUp : {
      if (topline>0) topline--;
      return;
    }
    case kDown : TreatKey(kBlindDown); break;
    case kUp : TreatKey(kBlindUp); break;
    case kPgdn : for (int i=0; i<LINESONSCREEN; i++) TreatKey(kBlindDown); break;
    case kPgup : for (int i=0; i<LINESONSCREEN; i++) TreatKey(kBlindUp); break;
    case kRight : if (fromleft<256) fromleft++; break;
    case kLeft : if (fromleft>0) fromleft--; break;
    case kHome : {
      if (fromleft>0) fromleft=0; else topline=0;
      break;
    }
    case kEnd  : {
      topline=lines.Size()-LINESONSCREEN+4; //+4: ??????????? But it works...
      if (topline<0) topline=0;
      break;
    }

    case kW : {
      DoWordWrap();
      break;
    }
    
    
    //Find
    case kF7 :
    case kAltF :
    case kF : {
      refresh=Find();
      refresh=true; //We want to get rid of the promptbox
      break;
    }

/*    case kS : {
      long int li=0;
      for(int i=0; i<lines.Size(); i++) li+=lines[i].size;
      MessageBox(MkStk(li,true));

    }*/

    //Find again
    case kAltG :
    case kG : {
      refresh=FindAgain();
      break;
    }

    case kF1 : ShowViewHelp(); break;
    
    default : refresh=false; //nouse key

  }
  if (refresh) Update();
}

string cViewWindow::GetTitle() {
  return "View: "+filename;
}

bool cViewWindow::Find() {
  if(InputWindow("Search for:",findstring,40)) {
    return FindAgain();
  }
  return false;
}

bool cViewWindow::FindAgain() {
  for(int i=topline+1; i<lines.Size(); i++) {
    int q=lines[i].find(findstring.c_str());//why doesn't it work with a normal string?
    if(q>-1) {
      topline=i;
      if ((topline+LINESONSCREEN) > lines.Size() ) {
        TreatKey(kEnd);
      }
      return true;
    }
  }
  return false;
}

/****************************************************************/

cViewProgramOutput::cViewProgramOutput(const string& progtorun):
  cViewWindow("Output from "+progtorun) {

  stdoutfile=popen(progtorun.c_str(),"r");
}


cViewProgramOutput::~cViewProgramOutput() {
  pclose(stdoutfile);
}

void cViewProgramOutput::Run() {

}

bool cViewProgramOutput::CheckOutput() {
  char buf[2048];
  fgets(buf,2048,stdoutfile);
  if (buf) {

  }
  return false;
}

/********************************************************************/
bool DedicatedView(const string& filename) {
  Wr(1,1,GetSpaces(ScreenW()),chYELLOW,cBLUE);
  Wr(1,ScreenH(),GetSpaces(ScreenW()-1),chYELLOW,cBLUE);
  cViewWindow vw(filename);
  if (vw.Error()) return false;
  do {
    tKey k=GetSomeKey();
    if (KeyIn(k,kF10,kESC,kAltX)) {
      //if (vw.ConfirmExit()) return ew.ChangedAtAll();
      return true;
    } else {
      vw.TreatKey(k);
    }
  } while(42); //Know the number? Of course you do...
  return false;
}
