/***************************************************************************
                          fm_window.h  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#ifndef _FM_WINDOW
#define _FM_WINDOW

#include "fm_str.h"
#include "fm_conio.h"
#include "fm_keyboard.h"
#include "array.h"
//#include <vector>

#define _WINDOW_DEFAULTBG cCYAN
#define _WINDOW_DEFAULTFG cBLACK

class cWindow {
  public:
    cWindow(int, int,const string& title="",int=_WINDOW_DEFAULTBG);
    ~cWindow();
    void FastWrite(int, int, const string&, int=_WINDOW_DEFAULTFG, int=-1);
    void Write(int, int, const string&, int=_WINDOW_DEFAULTFG, int=-1);
    void JustWrite(const string&, int=_WINDOW_DEFAULTFG, int=-1);
    void WriteOn(const string&, int=cBLACK, int=-1);
    void ChTitle(const string& s) {sUpleft=s;};
    void ChUpright(const string& s) {sUpright=s;};
    void ChDnleft(const string& s) {sDnleft=s;};
    void ChDnright(const string& s) {sDnright=s;};

    int GetXOffset() {return posx;};
    int GetYOffset() {return posy;};

    int GetHeight() {return height;};
    int GetWidth() {return width;};

    void GotoXY(int x, int y);
    void RedrawBox();
    void DrawHeader();
  protected:
    int lastx, lasty, lastlen;
    int width,height;
    int posx, posy;
    string sUpleft, sUpright, sDnleft, sDnright;
    sBox box;
    int bgcolor;

    void ReCalcBox();
};

class cInfoWindow {
  public:
    cInfoWindow(const string& s="",int bgcol=_WINDOW_DEFAULTBG) {
      title=s;
      bg=bgcol;
      width=s.length()+4;
      dnright=dnleft=upright="";
    }
    void AddLine(const string& s="",int col=_WINDOW_DEFAULTFG, int xpos=1) {
    lines+=s;
      //lines.push_back(s);
      colors+=col;
      x+=xpos;
      if ((int)s.length()+2>width) width=s.length()+2;
    }
    void operator += (const string& s) {AddLine(s);};
    tKey Show(bool=true);
    void SetDnRight(const string& s) {dnright=s;};
    void SetDnLeft(const string& s) {dnleft=s;};
    void SetUpRight(const string& s) {upright=s;};
  protected:
    cArray<string> lines;
    cArray<int> colors;
    cArray<int> x;
    int width, bg;
    string title;
    string dnright, dnleft, upright;
};

class cScrollingInfoWindow : public cInfoWindow {
  public:
    cScrollingInfoWindow(const string& s="", int bgcol=_WINDOW_DEFAULTBG) :
      cInfoWindow(s,bgcol) {;};
    tKey Show();
  protected:
    int height, top;
    void Redraw();
    cWindow* w;
};

/*class cInteractiveWindow:public cInfoWindow {
  public:
    cInteractiveWindow(const string& s="",int bgcol=_WINDOW_DEFAULTBG) {
      cInfoWindow(s,bgcol);
      topline=0;
    }
    tKey Show(tKey exitkey1,tKey exitkey2=kDummyKey,tKey exitkey3=kDummyKey) {;};
  protected:
    int topline;
};*/

//Shows a message at middle of screen
void ShowMessage(const string&,const string& title="", int=_WINDOW_DEFAULTBG/*,int=-1*/);

//Shows a message and waits for a keypress
void MessageBox(const string&,const string& title="", int=_WINDOW_DEFAULTBG/*,int=-1*/);

//Show an errormessage
void ErrorBox(const string&);

//Show error from ERRNO
void ShowErrorMessage();

//Prompt for something and return one of the specified keys
tKey PromptBox(const string& title, const string& message,
  tKey k1, tKey k2=kDummyKey, tKey k3=kDummyKey, tKey k4=kDummyKey, tKey=kDummyKey,tKey=kDummyKey,tKey=kDummyKey,tKey=kDummyKey);

//Shows a rotator in the corner
void ShowRotator();

//Returns a progressbar
string GetProgressBar(long int part, long int total,int length);

//Shows a progressbar at the center of the window
void ShowProgressWindow(long int, long int, int=40, const string& title="");

#endif //_FM_WINDOW


