/***************************************************************************
              main.cpp  -  mainly containing main-function ;-)
                             -------------------
    begin                : Sat Dec 18 22:13:24 CET 1999
    copyright            : (C) 1999-2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fm_defs.h"
#include "fm_debug.h"
#include "fm_vfs.h"
#include "fm_keyboard.h"
#include "fm_window.h"
#include "fm_init.h"
#include "fm_sysinfo.h"
#include "fm_console.h"
#include "fm_menu.h"
#include "fm_entry.h"
#include "fm_conf.h"
#include "sc_signals.h"
#include "sc_tipofday.h"
#include "fm_tree.h"

#ifdef _TARGET_LINUX_
  #include <unistd.h>
#endif

#include <iostream.h>

#define _MAX_CONSOLES 4

class cFM {
  public:
    //Constructor, creates the first console
    cFM() {
      Dbg("main: Init Sunshine Commander");
      FmTree=new cFmTree;
      Dbg("main: Creating first console");
      con[0]=new cConsole(1); //First console
      con[1]=NULL; con[2]=NULL; con[3]=NULL; //don't init others until used
      currentconsole=0;
      Dbg("main: Updating first console");
      Update();
    }

    //Destructor, destroys consoles with content
    ~cFM() {
      if (Configuration.gotolastdir) SetDir(con[currentconsole]->GetFullPath());
      Dbg("main: destroying consoles");
      for (int i=0; i<_MAX_CONSOLES; i++) {
        if (con[i]!=NULL) delete con[i];
      }
      delete FmTree;
      Dbg("main: Done Sunshine Commander");
    }

    //called when waiting for keyboardinput at toplevel, actually
    //we could do some nonpreemptive multitasking here...
    void Idle() {
      con[currentconsole]->Idle();
      #ifdef _TARGET_LINUX_
      // Fixes CPU time useage on Linux systems, need to know where usleep(1)
      // can be used before making this more general.
      usleep(1);
      #endif
    }

    //Updates current console
    void Update() {
      con[currentconsole]->Update();
    }

    //Main loop
    void Run() {
      //Cannot call it before as first console should be updated first
      if (Configuration.showtipofday) {
        ShowTipOfDay();
        Update();
      }
      
      tKey k;
      do {
         //The outcommented code here allowed instant update when resizing a window
      	 //in Xwin, unfortunately it also ate all CPU-time available :-(
      	 //Can somebody fix this?
         //Fixed,used SIGWINCH to determine when to redraw the screen
         //Still takes a lot of CPU time, but for a different reason
        do {
          #ifdef _TARGET_LINUX_
          if (_signal_windowresized) {
            RefreshConIO();
            Update();
            _signal_windowresized=false;
          }
          #endif
          Idle();
          
        } while (!IsKeyPressed());
        k=GetSomeKey();
        switch (k) {

          case kF6 : {
            ShowDriveInfo();
            ClearScreenBlack();
            Update();
            break;
          }
          case kShiftF1 :
          case kShiftF2 :
          case kShiftF3 :
          case kShiftF4 : {
            int i = Key_LastNumber()-1;
            if (currentconsole!=i) currentconsole=i; else break;
            if (con[i]==NULL) con[i]=new cConsole(i+1);
            Update();
            break;
          }

          default : {
            con[currentconsole]->TreatKey(k);
          }
        }
        if (KeyIn(k,kF10,kAltX,kQ)) {
          for(int i=0; i<_MAX_CONSOLES; i++) {
            if (con[i]!=NULL) { //if console is initialized
              con[i]->Update(); //Update it
              if (!con[i]->ConfirmExit()) {k=kDummyKey; break;} //and check if we can close it
            }
          }
        }
      } while (!KeyIn(k,kF10,kAltX,kQ));
      con[currentconsole]->SetOutDir();
    }
  private:
    cConsole* con[_MAX_CONSOLES];
    int currentconsole;
};

int main(int argc, char* argv[]) {
  StartUpSequence(argc,argv);

  cFM* its_alive=new cFM();
  its_alive->Run();
  delete its_alive;

  ShutDownSequence();
}
