#ifndef _SC_ACTION
#define _SC_ACTION

#include <string>
//#include <vector>
#include "array.h"
#include "fm_vfs.h"
#include "fm_window.h"
#include "stopwatch.h"

enum tActionType {acCopy, acMove, acErase, acPack, acUnpack, acAttr, acDefault};
enum tCheckAction {oaUndetermined, oaAlways,
                   oaNever, oaNewer,
                   oaDiffSize,oaCancel,
                   oaIgnore, oaResolve,
                   oaProcess, oaCopy, oaRetry, oaIgnoreAll};


typedef void (*tProgFunc)(long int, long int, int);

class cBaseAction {
  public:
    cBaseAction(const cDirEntry& de,const string& sname,tProgFunc pf) {
      entry=de;
      progfunc=pf;
      showname=sname;
    };
    string GetShowName() {return showname;};
    string GetResult() {return result;};
    void ClearResult() {result="";};
    void SetIgnore() {result="Ignore";};
    void SetFailure() {result="Failure";};
    void SetSuccess() {result="Success";};
    long GetSize() {return entry.size;};
    virtual bool Action()=0;
  protected:
    cDirEntry entry;
    string showname, result;
    tProgFunc progfunc;
};

class cActionProject {
  public:
    //cActionProject(const cDirEntry&, const string&, tActionType);
    cActionProject(const tDirContainer&, const string&, tActionType);
    ~cActionProject();
    bool Doit();
    bool Error() {return error;};
  private:
    string fromdir, destdir;
    long int filecount, filemeasure, totalmeasure, failures;
    bool error,allpressed,breaked;
    cWindow *w;
    tActionType atype;
    union {
      sAttrChanges attrch;
      sModeChanges modech;
    };
    
    cArray<cBaseAction*> actions;

    tCheckAction howto_subdirs;
    tCheckAction howto_overwrite;
    #ifdef _SYMLINKS_EXIST_
      tCheckAction howto_symlinks;
    #endif
    tCheckAction howto_failure;

    void Initialize();
    bool SetupAction();
    #ifdef _TARGETFLAVOR_DOS_
      bool SetAttrChanges();
    #endif
    bool SetModeChanges();
    bool ScanDir(const string&);
    void RegisterFilelist(const tDirContainer&,bool=true);
    void RegisterFile(const cDirEntry&, tActionType=acDefault);
    void UpdateList(int);
    
    bool ShouldI();

    //Is there a spaceproblem and what should I do about it?
    bool AskSpaceAction();
    void AskFailureAction();
    void AskDirAction();
    void AskOverwriteAction();
    #ifdef _SYMLINKS_EXIST_
      void AskSymlinkAction();
    #endif

    //friend class cBaseAction;
};


#endif //_SC_ACTION
