/***************************************************************************
                          sc_archive.cpp  -  description
                             -------------------
    begin                : Mon Mar 20 2000
    copyright            : (C) 2000 by Morten Poulsen
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//#include <string>
//#include <time.h>
#include "sc_archive.h"
//#include "unzip.h"
//#include "zip.h"
#include "fm_str.h"
#include "fm_vfs.h"
#include "fm_window.h"
#include "target.h"
#include "fm_str.h"
//#include "sc_file.h"
//#include "sc_action.h"
#include "fm_defs.h"
#include "arc_zip.h"
#include "arc_tgz.h"

void Add2Dots(tDirContainer& cont); //prototype, see end of file

void CheckForMissingDirRecord(tDirContainer& cont, const string& fullname, const sParsedDir& spd, tModeAttr modeattr) {
  string s=fullname;
  string arcdir=spd.arc_dirname;
  if (arcdir=="/") arcdir="";

  //Is it the same dircomplex?
  if (!MatchStart(arcdir, fullname)) return;

  //Skip to the current dir
  s=s.substr(arcdir.length(),999);
  if (s[0]=='/') s=s.substr(1,999);


  int i=s.find('/');
  if (i>-1) {
    //Isolate dirname
    s=s.substr(0,i);
  } else return; //OOpps, it was a file

  //Check whether we have added it allready
  for (i=0; i<cont.size(); i++) {
    if (cont[i].name==s) return; //Dir already there
  }

  //Dir not there, so we must make an entry
  cDirEntry tde;
  tde.MakeNormal(s,("{"+spd.fullname+"}"+AddSlash(arcdir)+s), modeattr,
     0,
     0,
     0);
  tde.isdir=true;
  cont+=tde;
}





bool FixDirsInArchive(string& name, const sParsedDir& spd) {
  string s=GetPartDir(StripSlash(name));
  name=GetPartName(StripSlash(name));
  if (s==spd.arc_dirname) return true; else return false;
}


///////////////////////General//////////////////////////////////

/*************************************************/
// Routines for dirscanning in archives
/*************************************************/

void Add2Dots(tDirContainer& cont) {
  cDirEntry tde;
  tde.MakeSpecial("..");
  cont+=tde;
  tde.MakeSpecial("...");
  cont+=tde;
}


bool ArchiveScan(tDirContainer& cont, const string& dir, bool onlydirs=false) {
  sParsedDir spd=PleaseParseDir(dir,true);
  if (spd.type!=dtArc) return false;
  if (spd.fileext==".ZIP") return ZIPScan(cont,spd,onlydirs);
  if (spd.fileext==".zip") return ZIPScan(cont,spd,onlydirs);
  if (spd.fileext==".tar.gz") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".TAR.GZ") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".tar.bz2") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".TAR.BZ2") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".tar") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".TAR") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".TGZ") return TarScan(cont,spd,onlydirs);
  if (spd.fileext==".tgz") return TarScan(cont,spd,onlydirs);
  return false;
}


cFile* GetArcFilePointer(const sParsedDir& spd) {
  if (spd.type!=dtArc) return NULL;
  if (spd.fileext==".ZIP") return        new cFileInZIP(spd);
  if (spd.fileext==".zip") return        new cFileInZIP(spd);
  if (spd.fileext==".tar.gz") return     new cFileInTar(spd);
  if (spd.fileext==".TAR.GZ") return     new cFileInTar(spd);
  if (spd.fileext==".tar.bz2") return     new cFileInTar(spd);
  if (spd.fileext==".TAR.BZ2") return     new cFileInTar(spd);
  if (spd.fileext==".tar") return        new cFileInTar(spd);
  if (spd.fileext==".TAR") return        new cFileInTar(spd);
  if (spd.fileext==".tgz") return        new cFileInTar(spd);
  if (spd.fileext==".TGZ") return        new cFileInTar(spd);
  return NULL;
}

bool UnpackWholeFile(const string& filename, const string& destdir, tProgFunc progress) {
  sParsedDir spd=PleaseParseDir(filename);
  if (spd.fileext==".ZIP") return     UnpackZIP(filename,destdir,progress);
  if (spd.fileext==".zip") return     UnpackZIP(filename,destdir,progress);
  if (spd.fileext==".tar.gz") return  UnpackTar(filename,destdir,progress);
  if (spd.fileext==".TAR.GZ") return  UnpackTar(filename,destdir,progress);
  if (spd.fileext==".tar.bz2") return  UnpackTar(filename,destdir,progress);
  if (spd.fileext==".TAR.BZ2") return  UnpackTar(filename,destdir,progress);
  if (spd.fileext==".tar") return     UnpackTar(filename,destdir,progress);
  if (spd.fileext==".TAR") return     UnpackTar(filename,destdir,progress);
  if (spd.fileext==".tgz") return     UnpackTar(filename,destdir,progress);
  if (spd.fileext==".TGZ") return     UnpackTar(filename,destdir,progress);
  return false;
}

bool ArcDoesDirExist(const string& dirname) {
  tDirContainer temp;
  return ArchiveScan(temp,dirname,true);
}

bool ArcDoesFileExist(const string& filename) {
  sParsedDir spd=PleaseParseDir(filename);
  cFile* f=GetArcFilePointer(spd);
  bool result=(f!=NULL); delete f;
  return result;
}

bool ArcDeleteFile(const string& name) {
  sParsedDir spd=PleaseParseDir(name);
  cFile* f=GetArcFilePointer(spd);
  if (f==NULL) return false;
  bool result=(f->Delete());
  delete f;
  return result;
}
