/***************************************************************************
     stopwatch.h  -  A simple stopwatch with a precision of seconds
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
  version 0.0.1 - Initial release
  version 0.0.2 - Fixed some stupid bugs
*/

#ifndef _SC_TIMER
#define _SC_TIMER

#include <time.h>
#include <string.h>
#include "fm_str.h"

class cStopwatch {
  public:
    cStopwatch() {Reset();};
    void Reset() {startvalue=SecsSinceMidnight();totalvalue=-1;};

    //Returns number of elapsed seconds
    int Elapsed() {
      int temp=(SecsSinceMidnight()-startvalue);
      if (temp<0) temp+=3600; //handle midnight, but not several days
      return temp;
    };

    //returns a formatted string of elapsed seconds
    string MkStrElapsed() {
      return FormatTime(Elapsed());
    };

    void SetTotalValue(long total) {totalvalue=total;};

    //return remaining time in seconds according to part/total
    long CalcRemainingTime(long part) {
      if (!totalvalue) return 0;
      double d=(Elapsed()*GetRatio(part));
      return (long(d));
    }

    //return a formatted string of remaining time
    string MkStrRemaining(long part) {
      if (!totalvalue) return "";
      return ("ETA: "+FormatTime(CalcRemainingTime(part)));
    }

    //return speed in units per sec. according to part/total
    unsigned long CalcSpeed(long part) {
      if (!totalvalue) return 0;
    }

    //returns a formatted string which assumes that some speed bytes are being
    //processed
    string MkStrByteSpeed(long part,const string& unit) {
      if (!totalvalue) return "";
      //not implemented yet
    }
  private:
    clock_t startvalue;
    long totalvalue;

    int SecsSinceMidnight() {
      time_t x; tm *t; time(&x); t = gmtime(&x);
      return ((t->tm_hour*3600)+(t->tm_min*60)+(t->tm_sec));
    }

    //returns the ratiopoart which is finished
    double GetRatio(long part) {
      if (!totalvalue) return -1;
      return ((part*1.0) / (totalvalue*1.0));
    }

    string FormatTime(long int i) {
      string s="";
      int h=(i / 3600); i=(i % 3600); //extract whole hours
      int m=(i / 60);   i=(i % 60);   //extract minuts
      int sec=i; //Paint yellow elephants on a pink, cloudy sky
      if (h) s+=(MkStr(h)+"h. ");
      if (m) s+=(MkStr(m)+"m. ");
      if (sec) s+=(MkStr(sec)+"s.");
      return s;
    }
};


#endif //_SC_TIMER
