#include "syntax_cpp.h"
#include "syntax_defs.h"

const int __default = 0;
const int __comment  = 2; //Comment spanning several lines
const int __commentl = 3; //Comment ending with a EOL
const int __string   = 4; //String spanning several lines
//notused
//const int __stringl  = 5; //string ending with a EOL
const int __ident   = 6;
const int __symbols = 7;
const int __literal = 8;
const int __preproc = 9;

string cpp_reserved_words[] = {
  //keywords taken from Bjarne Stroustrup's book
  //Rarely used ones outcommented...
  "string", //added by me ;,-)
  "and",/*"and_eq","asm","auto","bitand","bitor",*/
  "bool","break","case","catch","char","class",
  /*"compl",*/"const",/*"const_cast,*/"continue","default","delete",
  "do","double",/*"dynamic_cast",*/"else","enum",/*"explicit",*/
  "export","extern","false","float","for","friend",
  "goto","if","inline","int","long"/*,"mutable"*/,
  "namespace","new","not",/*"not_eq,*/"operator","or",
  /*"or_eq",*/"private","protected","public","register",/*"reinterpret_cast",*/
  "return","short","signed","sizeof","static",/*"static_cast"*/
  "struct","switch","template","this","throw","true",
  "try","typedef"/*,"typeid","typename"*/,"union","unsigned",
  "using","virtual","void",/*"volatile","wchar_y",*/"while",
  "xor",/*"xor_eq*/
  ""
};


class cParserCPP : cParser {
  public:
    cParserCPP() {
      temp=""; buf="";
      int i=0;
      while (cpp_reserved_words[i]!="") {
        reserved_words+=cpp_reserved_words[i];
        i++;
      }
      reserved_words+="";      
    };
    void SyntaxParse(cTextFile& lines);
/*  private:
    void AddBuf();*/
};


void cParserCPP::SyntaxParse(cTextFile& lines) {
  string ts="";
  int state=__default;
  char cc;
  do {
    do {cc=GetNextch(lines);} while (!cc); //disallow 0x00
    switch (Classify(cc)) {
      case __CHAR : {
        switch (state) {

          case __literal : {
            if (cc=='x') { //hexvalue
              Add(cc);
              break;
            }
            //else Syntax error, but we just fall through to...
          }
          
          case __symbols :
          case __default : {
            buf=cc;
            state=__ident;
            break;
          }
          case __ident : {
            buf=(buf+cc);
            break;
          }

          default : {
            Add(cc);
            break;
          }
        }
        break;
      };
    
      case __NUMBER : {
        switch (state) {
          case __default :
          case __symbols : {
            Add(cc,COL_LITERAL);
            state=__literal;
            break;
          }
          case __ident : {
            AddBuf(); //numbers can follow chars in idents
            break;
          }

          default : {
            Add(cc);
          }
        }
        break;
      }

      case __SYMBOL : {
        switch (state) {
          case __default :
          case __symbols: {
            switch (cc) {
              case '/' : {
                if (PeekNextch(lines)=='*') {
                  Add(cc,COL_COMMENT);
                  state=__comment;
                  break;
                }
                if (PeekNextch(lines)=='/') {
                  Add(cc,COL_COMMENT);
                  state=__commentl;
                  break;
                }
                Add(cc);
                break;
              }
              case '\"' : //String beginning
              case '\'' : {
                Add(cc,COL_STRING);
                state=__string;
                break;
              }
              //Fallthrough intensional
/*            }
          }

            switch (cc) {*/
              case '#' : {
                Add(cc,COL_PREPROC);
                state=__preproc;
                break;
              }
              default : {
                state=__symbols;
                Add(cc,COL_SYMBOL);
              }
            }
            break;
          }
          case __string : {
            switch (cc) {
              case '\"' : //String ending
              case '\'' : {
                Add(cc); AddCol(COL_DEFAULT);
                state=__default;
                break;
              }

              default : {
                Add(cc);
              }
            }
            break;
          }
          case __comment : {
            if (cc=='*' && PeekNextch(lines)=='/') { //end of comment?
              Add(cc); Add(GetNextch(lines));
              AddCol(COL_DEFAULT);
              state=__default;
              break;
            }
            Add(cc);
            break;
          }
          case __ident : {
            AddBuf(); //fallthrough intensional
            
          }
          case __literal : {
            state=__symbols;
            Add(cc,COL_SYMBOL);
            break;
          }

          default : {
            Add(cc);
          }
        }
        break;
      }

      case __SPACE : {
        switch (state) {
          case __ident : {
            state=__default;
            AddBuf(); //fallthrough intensional
            AddCol(COL_DEFAULT);
          }
          default : {
            Add(cc);
          }
        }
        
        break;
      }

      case __EOL : {
        switch (state) {
          case __comment :
          case __string : {
            //Do not reset to default at end of line
            break;
          }
          case __ident : {
            AddBuf(); //fallthrough intensional
          }

          default : {
            state=__default;
            currentcolor=COL_DEFAULT;
          }
        }
        lines[cline-1]=temp; //Replace with colorcoded line
//        int t=temp.length();
        temp="";
        //if (currentcolor!=COL_DEFAULT) {
        AddCol(currentcolor); //All lines start with colorcode
        //}
        break;
      }
      default : ; //at EOF do nothing particular
    }
  } while (cc!=_eof);
//  ts=lines[0];
  if (temp.length()>2) {
    lines[cline-1]=temp;
    //insertlast line!
  }
}


bool DoCPPSyntax(cTextFile& lines) {
  cParserCPP parser;
  parser.SyntaxParse(lines);
  return true;
}



/*bool IsReserved(const string& s) {
  int i=0;
  do {
    if (s==reserved_words[i]) return true;
    i++;
  } while (reserved_words[i]!="");
  return false;
}*/

